# $Id: pl3d.py,v 1.8 2003/09/30 23:43:47 travo Exp $
# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from shapetest import *
from yorick import *
from arrayfns import *


#  PL3D.PY
#  Viewing transforms and other aids for 3D plotting.
#
#  $Id: pl3d.py,v 1.8 2003/09/30 23:43:47 travo Exp $

#     Copyright (c) 1997.  The Regents of the University of California.
#                   All rights reserved.

"""
   General overview of module pl3d:

   (1) Viewing transform machinery.  Arguably the simplest model
       is the CAD/CAM notion that the object you see is oriented
       as you see it in the current picture.  You can then move
       it left, right, up, down, or toward or away from you,
       or you can rotate it about any of the three axes (horizontal,
       vertical, or out of the screen).  The xyz coordinates of the
       object remains unchanged throughout all of this, but this
       object coordinate system changes relative to the fixed
       xyz of the viewer, in which x is always to the right, y is
       up, and z is directed out of the screen.  Initially, the
       two coordinate systems coincide.

       rot3 (xangle,yangle,zangle)
         Rotate the object about viewer's x-axis by xangle, then
         about viewer's y-axis by yangle, then about viewer's
         z-axis by zangle
       mov3 (xchange,ychange,zchange)
         Move the object by the specified amounts.

       setz3 (zcamera)
         The "camera" is located at (0,0,zcamera) in the viewer's
         coordinate system, looking in the minus-z direction.
         Initially, zcamera is very large, and the magnification
         factor is correspondingly large, giving an isometric view.
         Decreasing zcamera makes the perspective more extreme.
         If parts of the object are behind the camera, strange things
         may happen.

       undo3 ()
       undo3 (n)
         Undo the last N (default 1) viewpoint commands (rot3, mov3,
         or setz3).  Up to 100 viewpoint changes are remembered.
       viewpoint= save3()
       ...
       restore3 (viewpoint)
         The current viewpoint transformation can be saved and later
         restored.

       gnomon (on_off)
         Toggle the gnomon (a simple display showing the orientation
         of the xyz axes of the object).
"""

#  ------------------------------------------------------------------------


def set_draw3_ ( n ) :

   """
   set_draw3_ ( 0 | 1 ) is used to set the global draw3_,
   which controls whether the function draw3 actually shows a drawing.
   """
   
   global _draw3
   _draw3 = n

def setrot3_ (x) :

   # ZCM 2/21/97 change reflects the fact that I hadn't realized
   # that car and cdr, as functions, return the item replaced.

   global _draw3_list
   oldx = _draw3_list [0]
   _draw3_list [0] = x
   undo3_set_ (setrot3_, oldx)

def rot3 (xa = 0., ya = 0., za = 0.) :

   """
   rot3 (xa, ya, za)
   rotate the current 3D plot by XA about viewer's x-axis,
   YA about viewer's y-axis, and ZA about viewer's z-axis.
   SEE ALSO: orient3, mov3, aim3, setz3, undo3, save3, restore3, light3
   """

   x = array ([1.,0.,0.], Float)
   y = array ([0.,1.,0.], Float)
   z = array ([0.,0.,1.], Float)
   [x, y] = rot3_ (za, x, y)
   [z, x] = rot3_ (ya, z, x)
   [y, z] = rot3_ (xa, y, z)
   # n. b. matrixMultiply has the unfortunate effect of destroying
   # the matrix that calls it.
   gr3 = array (getrot3_ (), copy = 1)
   setrot3_ (transpose (dot (transpose (gr3), array ( [x, y, z]))))

def rot3_ (a, x, y) :
   ca = cos (a)
   sa = sin (a)
   return [multiply (ca, x) + multiply (sa, y), multiply (-sa, x) + multiply (ca, y)]

def mov3 ( xa = 0., ya = 0., za = 0. ) :

   """
   mov3 ( [xa [, ya [, za]]])
   move the current 3D plot by XA along the viewer's x axis,
   YA along the viewer's y axis, and ZA along the viewer's z axis.
   SEE ALSO: rot3, orient3, setz3, undo3, save3, restore3, light3
   """

   gr = dot (transpose (gr), transpose (xa))
   setorg3_ ( getorg3_ () - gr) 

def aim3 ( xa = 0., ya = 0., za = 0. ) :

   """
   aim3 ( [xa [, ya [, za]]])
   move the current 3D plot to put the point (XA, YA, ZA) in object
   coordinates at the point (0, 0, 0) -- the aim point -- in the
   viewer's coordinates. If any of the XA, YA, or ZA is nil, it defaults
   SEE ALSO: mov3, rot3, orient3, setz3, undo3, save3, restore3, light3
   """

   setorg3_ (x)

_ZcError = "ZcError"

def setz3 ( zc = None ) :

   """
   setz3 ( [zc] )
   Set the camera position to z = ZC (x = y = 0) in the viewer's coordinate
   system. If zc is None, set the camera to infinity (default).
   SEE ALSO: rot3, orient3, undo3, save3, restore3, light3
   """

   if not is_scalar (zc) :
      raise _ZcError, "camera position must be scalar."

   setzc3_ (zc)

def orient3 ( ** kw ) :

   """
   orient3 ( [phi = val1, theta = val2] )
   Set the orientation of the object to (PHI, THETA). Orientations
   are a subset of the possible rotation matrices in which the z axis
   of the object appears vertical on the screen (that is, the object
   z axis projects onto the viewer y axis). The THETA angle is the
   angle from the viewer y axis to the object z axis, positive if
   the object z axis is tilted towards you (toward viewer +z). PHI is
   zero when the object x axis coincides with the viewer x axis. If
   neither PHI nor THETA is specified, PHI defaults to - pi / 4 and
   THETA defaults to pi / 6. If only PHI is specified, THETA remains
   unchanged, unless the current THETA is near pi / 2, in which case
   THETA returns to pi / 6, or unless the current orientation does
   not have a vertical z axis, in which case THETA returns to its
   default.
   Unlike rot3, orient3 is not a cumulative operation.
   """
   # Notes with regard to global variables: (ZCM 2/21/97)
   # _orient3_phi, _orient3_theta, the default orientation angles,
   #    are known and referred to only in this routine. I have started
   #    them with an underscore, too, to make them inaccessible
   #    from outside this module.
   # phi and theta need not be global here since they are recalculated
   #    each time this routine is called.
   
   global _orient3_phi, _orient3_theta
   try :
      dummy = _orient3_theta
   except :
      _orient3_theta = pi / 6.

   try :
      dummy = _orient3_phi
   except :
      _orient3_phi = - pi / 4.

   if kw.has_key ("phi") and kw ["phi"] == None :
      kw ["phi"] = _orient3_phi
   if kw.has_key ("theta") and kw ["theta"] == None :
      kw ["theta"] = _orient3_theta
   if not kw.has_key ("phi") and not kw.has_key ("theta") :
      phi = _orient3_phi
      theta = _orient3_theta
   elif not kw.has_key ("phi") or not kw.has_key ("theta") :
      gr3 = array (getrot3_ (), copy = 1)
      z = dot (transpose (gr3), array ( [0., 0., 1.]))
      if abs (z [0]) > 1.e-6 :
         # object z-axis not aligned with viewer y-axis
         if not kw.has_key ("theta") :
            theta = _orient3_theta
            phi = kw ["phi"]
         else :
            phi = _orient3_phi
            theta = kw ["theta"]
      elif not kw.has_key ("theta") :
         phi = kw ["phi"]
         if (abs (z [1]) < 1.e-6) :
            theta = _orient3_theta
         else :
            theta = arctan2 (z [2], z [1])
      else :
         theta = kw ["theta"]
         y = array ( [0., z [2], -z [1]])
         x = dot (transpose (gr3), array ( [1., 0., 0.]))
         phi = arctan2 (sum (y * x), x [0])
   else :
      phi = kw ["phi"]
      theta = kw ["theta"]

   x = array ( [1., 0., 0.],  Float)
   y = array ( [0., 1., 0.],  Float)
   z = array ( [0., 0., 1.],  Float)
   [y, z] = rot3_ (theta, y, z)
   [z, x] = rot3_ (phi, z, x)
   setrot3_ (array ( [x, -z, y],  Float))

import copy

def save3 ( ) :

   """
   view = save3 ( )
     Save the current 3D viewing transformation and lighting.
     Actually, this doesn't save anything; it returns a copy
     of the current 3D viewing transformation and lighting, so
     that the user can put it aside somewhere.
   SEE ALSO: restore3, rot3, mov3, aim3, light3
   """

   return _draw3_list [0:_draw3_n]

def restore3 ( view = None ) :

   """
   restore3 ( view )
   Restore a previously saved 3D viewing transformation and lighting.
   If view is missing, rotate object to viewer's coordinate system.
   SEE ALSO: restore3, rot3, mov3, aim3, light3
   """

   global _draw3_list, _draw3_view, _light3_list, _draw3_n

   if view != None :
      view = view [0:len (view)] # Copies view
   else :
      view = _draw3_view + _light3_list
   old = _draw3_list [0:_draw3_n]
   _draw3_list = view [0:_draw3_n] + _draw3_list [_draw3_n:]
   undo3_set_ (restore3, old)

_AmbientError = "AmbientError"
_DiffuseError = "DiffuseError"
_LightingError = "LightingError"

def light3 ( * kw, ** kwds ) :

   """
   light3 (ambient=a_level,
                    diffuse=d_level,
                    specular=s_level,
                    spower=n,
                    sdir=xyz)
     Sets lighting properties for 3D shading effects.
     A surface will be shaded according to its to its orientation
     relative to the viewing direction.

     The ambient level A_LEVEL is a light level (arbitrary units)
     that is added to every surface independent of its orientation.

     The diffuse level D_LEVEL is a light level which is proportional
     to cos(theta), where theta is the angle between the surface
     normal and the viewing direction, so that surfaces directly
     facing the viewer are bright, while surfaces viewed edge on are
     unlit (and surfaces facing away, if drawn, are shaded as if they
     faced the viewer).

     The specular level S_LEVEL is a light level proportional to a high
     power spower=N of 1+cos(alpha), where alpha is the angle between
     the specular reflection angle and the viewing direction.  The light
     source for the calculation of alpha lies in the direction XYZ (a
     3 element vector) in the viewer's coordinate system at infinite
     distance.  You can have ns light sources by making S_LEVEL, N, and
     XYZ (or any combination) be vectors of length ns (3-by-ns in the
     case of XYZ).  (See source code for specular_hook function
     definition if powers of 1+cos(alpha) aren't good enough for you.)

     With no arguments, return to the default lighting.

   EXAMPLES:
     light3 ( diffuse=.1, specular=1., sdir=[0,0,-1])
       (dramatic "tail lighting" effect)
     light3 ( diffuse=.5, specular=1., sdir=[1,.5,1])
       (classic "over your right shoulder" lighting)
     light3 ( ambient=.1,diffuse=.1,specular=1.,
             sdir=[[0,0,-1],[1,.5,1]],spower=[4,2])
       (two light sources combining previous effects)
   SEE ALSO: rot3, save3, restore3
   """

   global _draw3_list, _draw3_nv
   if len (kw) > 0 : kwds = kw [0]
   old = _draw3_list [_draw3_nv:] [0:5]
   flags = 0
   if kwds.has_key ("ambient") and kwds ["ambient"] != None :
      ambient = kwds ["ambient"]
      if not is_scalar (ambient) :
         raise _AmbientError, "ambient light level must be scalar."
      flags = flags | 1
      _draw3_list [_draw3_nv] = ambient
   if kwds.has_key ("diffuse") and kwds ["diffuse"] != None :
      diffuse = kwds ["diffuse"]
      if not is_scalar (diffuse) :
         raise _DiffuseError, "diffuse light level must be scalar."
      flags = flags | 2
      _draw3_list [_draw3_nv + 1 ] = diffuse

   if kwds.has_key ("specular") and kwds ["specular"] != None :
      specular = kwds ["specular"]
      flags = flags | 4
   else :
      specular = _draw3_list [_draw3_nv + 2]
   if kwds.has_key ("spower") and kwds ["spower"] != None :
      spower = kwds ["spower"]
      flags = flags | 8
   else :
      spower = _draw3_list [_draw3_nv + 3]
   if kwds.has_key ("sdir") and kwds ["sdir"] != None :
      sdir = kwds ["sdir"]
      dims = shape (sdir)
      if dims == 0 or len (dims) == 2 and dims [1] != 3 :
         raise _LightingError, \
            "lighting direction must be 3 vector or ns-by-3 array."
      flags = flags | 16
   else :
      sdir = _draw3_list [_draw3_nv + 4]
   if flags & 28 :
      if flags & 4 : _draw3_list [_draw3_nv + 2] = specular
      if flags & 8 : _draw3_list [_draw3_nv + 3] = spower
      if flags & 16 : _draw3_list [_draw3_nv + 4] = sdir
   if not flags :
      _draw3_list [_draw3_nv: _draw3_nv + 5] = _light3_list [0:5]
   undo3_set_ (light3_, old)

def light3_ (arg) :
   global _draw3_list, _draw3_nv
  
   _draw3_list [_draw3_nv:_draw3_nv + 5] = arg  [0:5]
   
def get3_light (xyz, * nxyz) :

   """
   get3_light(xyz, nxyz)
      or get3_light(xyz)

     return 3D lighting for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len (NXYZ) long; in the second case, the
     return value is (ni-1)-by-(nj-1).

     The parameters of the lighting calculation are set by the
     light3 function.

     SEE ALSO: light3, set3_object, get3_normal, get3_centroid
     """

   global _draw3_list, _draw3_nv
   list = _draw3_list [_draw3_nv:]
   ambient = list [0]
   diffuse = list [1]
   specular = list [2]
   spower = list [3]
   sdir = list [4]

   if len (nxyz) == 0 :
      normal = get3_normal (xyz)
   else :
      normal = get3_normal (xyz, nxyz [0])

   zc = getzc3_ ( )
   if ( not zc ) :
      view = array ( [0., 0., 1.],  Float)
   elif len (nxyz) == 0 :
      view = array ( [0., 0., zc],  Float) - get3_centroid (xyz)
   else :
      view = array ( [0., 0., zc],  Float) - get3_centroid (xyz, nxyz [0]) 
      m1 = \
         sqrt ( sum (view * view))
      if m1 == 0. : m1 = 1.
      view = view / m1

   nv = normal [0, ...] * view [0] + normal [1, ...] * view [1] +  \
      normal [2, ...] * view [2]
   light = ambient + diffuse * abs (nv)
   if specular != 0. :
      sv = transpose (transpose (sdir) / sqrt (sum (transpose (sdir*sdir))))
      sv = dot (sv, view)
      if len (shape (sdir)) == 1 :
         sn = sum(array([sdir[0]*normal[0],sdir[1]*normal[1],
                         sdir[2]*normal[2]]))
         ####### I left out the specular_hook stuff.
         m1 = maximum (sn * nv -0.5 * sv + 0.5, 1.e-30)
         m1 = m1 ** spower
         light = light + (specular * m1)
      elif len (shape (sdir)) >= 2 :
         # multiple light sources
         nsrc = len (shape (sdir))
         for i in range (nsrc) :
             sn = sum(array([sdir[i,0]*normal[0],sdir[i,1]*normal[1],
                         sdir[i,2]*normal[2]]))
             m1 = maximum (sn * nv -0.5 * sv [i] + 0.5, 1.e-30) ** spower [i]
             light = light + specular * m1
   return light

def get3_normal (xyz, *nxyz) :

   """
     get3_normal(xyz, nxyz)
         or get3_normal(xyz)

     return 3D normals for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len(NXYZ)-by-3; in the second case, the
     return value is (ni-1)-by-(nj-1)-by-3.

     The normals are constructed from the cross product of the lines
     joining the midpoints of two edges which as nearly quarter the
     polygon as possible (the medians for a quadrilateral).  No check
     is made that these not be parallel; the returned "normal" is
     [0,0,0] in that case.  Also, if the polygon vertices are not
     coplanar, the "normal" has no precisely definable meaning.

     SEE ALSO: get3_centroid, get3_light
     """

   if len (nxyz) == 0 :
      # if no polygon list is given, assume xyz is 2D mesh
      # form normal as cross product of medians
      m1 = dif_ (zcen_ (xyz, 1), 2)
      m2 = zcen_ (dif_ (xyz, 1), 2)
   else :
      # with polygon list, more elaborate calculation required
      # (1) frst subscripts the first vertex of each polygon
      frst = cumsum (nxyz [0]) - nxyz [0]

      # form normal by getting two approximate diameters
      # (reduces to above medians for quads)
      # (2) compute midpoints of first three sides
      n2 = (nxyz [0] + 1) / 2
      c0 = (take(xyz, frst) + take(xyz, frst + 1)) / 2.
      i = frst + n2 - 1
      c1 = (take(xyz, i) + take(xyz, i + 1)) / 2.
      i = n2 / 2
      c2 = (take(xyz, frst + i) + take(xyz, frst + (i + 1) % nxyz [0])) / 2.
      i = minimum (i + n2, nxyz [0]) - 1
      c3 = (take(xyz, frst + i) + take(xyz, frst + (i + 1) % nxyz [0])) / 2.
      m1 = c1 - c0
      m2 = c3 - c2

   # poly normal is cross product of two medians (or diameters)
   # normal = m1; I had to reverse the sign.
   if len (shape (xyz)) == 3 :
      n1 = m1 [2, :] * m2 [1, :] - \
                             m1 [1, :] * m2 [2, :]
      n2 = m1 [0, :] * m2 [2, :] - \
                             m1 [2, :] * m2 [0, :]
      n3 = m1 [1, :] * m2 [0, :] - \
                             m1 [0, :] * m2 [1, :]
   else :
      n1 = m1 [:, 2] * m2 [:, 1] - \
                             m1 [:, 1] * m2 [:, 2]
      n2 = m1 [:, 0] * m2 [:, 2] - \
                             m1 [:, 2] * m2 [:, 0]
      n3 = m1 [:, 1] * m2 [:, 0] - \
                             m1 [:, 0] * m2 [:, 1]
   m1 = sqrt (n1 ** 2 + n2 **2 + n3 **2)
   m1 = m1 + equal (m1, 0.0)
   normal = array([n1 / m1, n2 / m1, n3 / m1])

   return normal

def get3_centroid (xyz, * nxyz) :

   """
     get3_centroid(xyz, *nxyz)
         or get3_centroid(xyz)

     return 3D centroids for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len(NXYZ) in length; in the second case, the
     return value is (ni-1)-by-(nj-1)-by-3.

     The centroids are constructed as the mean value of all vertices
     of each polygon.

     SEE ALSO: get3_normal, get3_light
   """

   if len (nxyz) == 0 :
      # if no polygon list is given, assume xyz is 2D mesh
      centroid = zcen_ (zcen_ (xyz, 1), 0)
   else :
      # with polygon list, more elaborate calculation required
      last = cumsum (nxyz [0])
      list = histogram (1 + last) [0:-1]
      list = cumsum (list)
      k = len (nxyz [0])
      l = shape (xyz) [0]
      centroid = zeros ( (k, 3))
      centroid [0:k, 0] = histogram (list, xyz [0:l,0])
      centroid [0:k, 1] = histogram (list, xyz [0:l,1])
      centroid [0:k, 2] = histogram (list, xyz [0:l,2])
      fnxyz = array (nxyz [0], Float )
      centroid = centroid / fnxyz
   return centroid

_Get3Error = "Get3Error"

def get3_xy (xyz, *flg) :

   """
     get3_xy (xyz)
         or get3_xy(xyz, 1)

     Given anything-by-3 coordinates XYZ, return X and Y in viewer's
     coordinate system (set by rot3, mov3, orient3, etc.).  If the
     second argument is present and non-zero, also return Z (for use
     in sort3d or get3_light, for example).  If the camera position
     has been set to a finite distance with setz3, the returned
     coordinates will be tangents of angles for a perspective
     drawing (and Z will be scaled by 1/zc).
     Unlike the Yorick version, this function returns a 3-by-anything
     array of coordinates.
     Actually, what it returns is a 3-by-anything python array, whose
     0th element is the x array, whose 1th element is the y array, and
     whose 2th element is the z array if asked for.
     I believe that x, y, and z can be either 1d or 2d, so this
     routine is written in two cases.

   """

   # rotate and translate to viewer's coordinate system
   shp = shape (xyz)
   if len (shp) == 3:
      # 2d mesh case is much more complex than in Yorick
      (k, l) = shp [1:3]
      go3_ = getorg3_ ()
      # Unwind xyz
      xx = ravel (xyz [0])
      yy = ravel (xyz [1])
      zz = ravel (xyz [2])
      tmpxyz = array ( [xx, yy, zz])
      gr3 = array (getrot3_ (), copy = 1)
      tmpxyz = dot (transpose (gr3),
         tmpxyz - array ( [ [go3_ [0]], [go3_ [1]], [go3_ [2]]]))
##    xx = transpose (reshape (ravel (tmpxyz [0]), (k,l)))
##    yy = transpose (reshape (ravel (tmpxyz [1]), (k,l)))
##    zz = transpose (reshape (ravel (tmpxyz [2]), (k,l)))
      xx = (reshape (ravel (tmpxyz [0]), (k,l)))
      yy = (reshape (ravel (tmpxyz [1]), (k,l)))
      zz = (reshape (ravel (tmpxyz [2]), (k,l)))
      tmpxyz = array ( [xx, yy, zz])
   elif len (shp) == 2:
      go3_ = getorg3_ ()
      lm = array (getrot3_ (), copy = 1)
      rm = (xyz - array ( [ go3_ [0], go3_ [1], go3_ [2]]))
      tmpxyz = dot (rm, lm)
   else:
      raise _Get3Error, "xyz has a bad shape: " + `shp`

   # do optional perspective projection 
   zc = getzc3_ ()
   if zc != None :
      if len (shp) == 2 :
         z = tmpxyz [:, 2]
         zc = maximum (zc - z, 1.e-35)     # protect behind camera, avoid zero divide
         tmpxyz [:, 0] = tmpxyz [:, 0] / zc
         tmpxyz [:, 1] = tmpxyz [:, 1] / zc
         if len (flg) != 0 and flg [0] != 0 :
            tmpxyz [:, 2] = tmpxyz [:, 2] / zc
      elif len (shp) == 3 :
         z = tmpxyz [:,:, 2]
         zc = maximum (zc - z, 1.e-35)     # protect behind camera, avoid zero divide
         tmpxyz [:,:, 0] = tmpxyz [:,:, 0] / zc
         tmpxyz [:,:, 1] = tmpxyz [:,:, 1] / zc
         if len (flg) != 0 and flg [0] != 0 :
            tmpxyz [:,:, 2] = tmpxyz [:,:, 2] / zc
   return tmpxyz

_UndoError = "UndoError"

_in_undo3 = 0
_undo3_list = []

def undo3 (n = 1) :

   """
     undo3 ()
         or undo3 (n)
     Undo the effects of the last N (default 1) rot3, orient3, mov3, aim3,
     setz3, or light3 commands.
   """

   global _in_undo3, _undo3_list
   n = 2 * n
   if n < 0 or n > len (_undo3_list) :
      raise _UndoError, "not that many items in undo list"
   _in_undo3 = 1     # flag to skip undo3_set_
   # perhaps should save discarded items in a redo list?
   use_list = undo3_list [-n:]
   undo3_list = undo3_list [:-n]
   while n > 0 :
      fnc = use_list_ [0]
      del use_list_ [0]
      arg = use_list_ [0]
      del use_list_ [0]
      fnc (arg)
      n = n - 2
   _in_undo3 = 0
   draw3_trigger ( )

def set3_object (fnc, arg) :

   """
     set3_object (drawing_function, [arg1,arg2,...])

     set up to trigger a call to draw3, adding a call to the
     3D display list of the form:

        DRAWING_FUNCTION ( [ARG1, ARG2, ...]))

     When draw3 calls DRAWING_FUNCTION, the external variable draw3_
     will be non-zero, so DRAWING_FUNCTION can be written like this:

     def drawing_function(arg) :

       if (draw3_) :
          arg1= arg [0]
          arg1= arg [1]
          ...
          ...<calls to get3_xy, sort3d, get3_light, etc.>...
          ...<calls to graphics functions plfp, plf, etc.>...
          return

       ...<verify args>...
       ...<do orientation and lighting independent calcs>...
       set3_object (drawing_function, [arg1,arg2,...])

   SEE ALSO: get3_xy, get3_light, sort3d
   """

   global _draw3_list
   _draw3_list = _draw3_list + [fnc, arg]
   draw3_trigger ()

def setorg3_ ( x ) :
   # ZCM 2/21/97 change reflects the fact that I hadn't realized
   # that car and cdr, as functions, return the item replaced.
   global _draw3_list
   oldx = _draw3_list [1]
   _draw3_list [1] = x
   undo3_set_ ( setorg3_,  oldx)

def setzc3_ (x) :
   # ZCM 2/21/97 change reflects the fact that I hadn't realized
   # that car and cdr, as functions, return the item replaced.
   global _draw3_list
   oldx = _draw3_list [2]
   _draw3_list [2] = x
   undo3_set_ ( setzc3_,  oldx)

def getrot3_ () :
   return _draw3_list [0]

def getorg3_ () :
   return _draw3_list [1]

def getzc3_ () :
   return _draw3_list [2]

def undo3_set_ (fnc, arg) :
   global _undo3_list, _in_undo3, _undo3_limit
   # arg = copy.deepcopy (arg)
   if not _in_undo3 :
      if len (_undo3_list) >= 2 * _undo3_limit :
         _undo3_list = _undo3_list [0:2 * _undo3_limit - 2]
      _undo3_list = [fnc, arg] + _undo3_list
   draw3_trigger ( )

_in_undo3 = 0         # ??????????????
_in_undo3 = 100

def do_nothing ( ) :
   pass
   return

def clear_idler ( ) :
   _idler = do_nothing ( )

def set_idler ( fnc ) :
   global _idler
   _idler = fnc

def call_idler ( ) :
   global _idler
   _idler ( )

def _draw3_idler ( ) :
   # I have added orientation and limits to this because they may not
   # have been set by a previous command. If the user doesn't like this,
   # he/she can write his/her own idler. (ZCM 7/1/97)
   global _default_gnomon
   orient3 ()
   if current_window () == -1 :
      window3 (0)
   else :
      window3 (current_window ())
   gnomon (_default_gnomon)
   lims = draw3 (1)
   if lims == None :
      return
   else :
      limits (lims [0], lims [1], lims [2], lims [3])

def set_default_idler ( ) :
   set_idler (_draw3_idler)

set_default_idler ( )

_draw3_changes = None

def set_multiple_components ( n = 0 ) :
   global _multiple_components
   _multiple_components = n

set_multiple_components (0)

def has_multiple_components () :
   global _multiple_components
   return _multiple_components

def draw3_trigger ( ) :
   "arrange to call draw3 when everything else is finished"
   global _draw3_changes
   global _draw3_idler
   set_idler ( _draw3_idler )
   _draw3_changes = 1

def clear3 ( ) :
   "clear3 ( ) : Clear the current 3D display list."
   global _draw3_list, _draw3_n
   _draw3_list [_draw3_n:] = []
   set_multiple_components (0)

def window3 ( * n , **kw ) :

   """
   window3 ( ) or window3 (n)
   initialize style="nobox.gs" window for 3D graphics
   """

   if kw.has_key ("dump") :
      dump = kw ["dump"]
   else :
      dump = 0
   if kw.has_key ("hcp") :
      if len (n) == 0 :
         window (wait=1, style="nobox.gs", legends=0, hcp=kw ["hcp"],
            dump = dump)
         hcpon ()
      else :
         window (n [0], wait=1, style="nobox.gs", legends=0, hcp=kw ["hcp"],
            dump = dump)
         hcpon ()
   else :
      if len (n) == 0 :
         window (wait=1, style="nobox.gs", legends=0)
      else :
         window (n [0], wait=1, style="nobox.gs", legends=0)

def sort3d (z, npolys) :

   """
   sort3d(z, npolys)
     given Z and NPOLYS, with len(Z)==sum(npolys), return
     a 2-element list [LIST, VLIST] such that Z[VLIST] and NPOLYS[LIST] are
     sorted from smallest average Z to largest average Z, where
     the averages are taken over the clusters of length NPOLYS.
     Within each cluster (polygon), the cyclic order of Z[VLIST]
     remains unchanged, but the absolute order may change.

     This sorting order produces correct or nearly correct order
     for a plfp command to make a plot involving hidden or partially
     hidden surfaces in three dimensions.  It works best when the
     polys form a set of disjoint closed, convex surfaces, and when
     the surface normal changes only very little between neighboring
     polys.  (If the latter condition holds, then even if sort3d
     mis-orders two neighboring polys, their colors will be very
     nearly the same, and the mistake won't be noticeable.)  A truly
     correct 3D sorting routine is impossible, since there may be no
     rendering order which produces correct surface hiding (some polys
     may need to be split into pieces in order to do that).  There
     are more nearly correct algorithms than this, but they are much
     slower.
   SEE ALSO: get3_xy
   """

   # first compute z, the z-centroid of every poly
   # get a list the same length as x, y, or z which is 1 for each
   # vertex of poly 1, 2 for each vertex of poly2, etc.
   # the goal is to make nlist with histogram(nlist)==npolys
   nlist = histogram(cumsum (npolys)) [0:-1]
   nlist = cumsum (nlist)
   # now sum the vertex values and divide by the number of vertices
   z = histogram (nlist, z) / npolys

   # sort the polygons from smallest z to largest z
   list = index_sort (z)
   # next, find the list which sorts the polygon vertices
   # first, find a list vlist such that sort(vlist) is above list
   vlist = zeros (len (list), Int)
   array_set (vlist, list, arange (len (list), typecode = Int))
   # then reset the nlist values to that pre-sorted order, so that
   # sort(nlist) will be the required vertex sorting list
   nlist = take(vlist, nlist)
   # the final hitch is to ensure that the vertices within each polygon
   # remain in their initial order (sort scrambles equal values)
   # since the vertices of a polygon can be cyclically permuted,
   # it suffices to add a sawtooth function to a scaled nlist to
   # produce a list in which each cluster of equal values will retain
   # the same cyclic order after the sort
   # (note that the more complicated msort routine would leave the
   #  clusters without even a cyclic permutation, if that were
   #  necessary)
   n1max = max (npolys)    # this must never be so large that
                           # numberof(npolys)*nmax > 2e9  
   nmax = n1max * ones (len (nlist), Int)
   vlist = index_sort (nmax * nlist +
      arange (len (nlist), typecode = Int) % n1max)
   #         primary sort key ^            secondary key  ^
   return [list, vlist]

_square = 1 # Global variable which tells whether to force equal axes
_xfactor = 1.
_yfactor = 1. # These globals enable one to scale one or both axes up or down

def get_factors_ ( ) :
   return [_xfactor, _yfactor]

def get_square_ ( ) :
   global _square
   return _square

def limits_ (square = 0, yfactor = 1., xfactor = 1.) :
   global _square, _xfactor, _yfactor
   _square = square
   _xfactor = xfactor
   _yfactor = yfactor

def draw3 (called_as_idler = 0, lims = None) :

   """
      draw3 (called_as_idler = 0, lims = None):
   Draw the current 3d display list.
   Ordinarily triggered automatically when the drawing changes.
   """

   global _draw3, _draw3_changes, _draw3_list, _draw3_n, _gnomon
   if _draw3_changes :
      if called_as_idler :
         fma ( )
      
      # the first _draw3_n elements of _draw3_list are the viewing
      # transforms, lighting, etc.
      # thereafter, elements are (function, argument-list) pairs
      # the _draw3 flag alerts the functions that these are the draw
      # calls rather than the interactive setup calls
      set_draw3_ (1)
      list = _draw3_list [_draw3_n:]
      no_lims = lims == None
      first = 1
      # ZCM Feb. 1997: Because Gist command 'limits' seems to
      # misbehave and be timing dependent, I have added the kludge
      # below, which seems to make things work.
      while list != [] :
         fnc = list [0]
         if no_lims :
            if (first) :
               lims = fnc (list [1])
               first = 0
            else :
               fv = fnc (list [1])
               if fv != None and lims != None :
                  lims = [min (fv [0], lims [0]),
                          max (fv [1], lims [1]),
                          min (fv [2], lims [2]),
                          max (fv [3], lims [3])]
               elif fv != None :
                  lims = fv
         else :
            fnc (list [1])
         list = list [2:]
      if _gnomon :
         _gnomon_draw ( )
      _draw3_changes = None
      set_draw3_ (0)
      return lims
#     _draw3 = 0

try :
   dummy = _draw3_view
except :
   _draw3_view = [array ([[1, 0, 0], [0, 1, 0], [0, 0, 1]]), [0., 0., 0.], None]
_draw3_nv = len (_draw3_view)

try :
   dummy = _draw3
except :
   set_draw3_ (0)

def get_draw3_ ( ) :
   global _draw3
   return _draw3

try :
   dummy = _light3_ambient
except :
   _light3_ambient = 0.2

try :
   dummy = _light3_diffuse
except :
   _light3_diffuse = 1.0

try :
   dummy = _light3_specular
except :
   _light3_specular = 0.0

try :
   dummy = _light3_spower
except :
   _light3_spower = 2

try : 
   dummy = _light3_sdir
except :
   _light3_sdir = array ( [1.0, 0.5, 1.0]) / sqrt(2.25)

_light3_list = [_light3_ambient, _light3_diffuse, _light3_specular,
                _light3_spower, _light3_sdir]


_draw3_list = _draw3_view + _light3_list
_draw3_n = len (_draw3_list)

def get_draw3_list_ ( ) :
   global _draw3_list
   return _draw3_list

def get_draw3_n_ ( ) :
   global _draw3_n
   return _draw3_n

try :
   dummy = _gnomon
except :
   _gnomon = 0

def set_default_gnomon ( * n ) :
   # The default gnomon value is used when _draw3 is nonzero, i. e.,
   # when a plot is actually done after every plot call.
   global _default_gnomon
   if len (n) > 0 :
      _default_gnomon = n
   else :
      _default_gnomon = 0

set_default_gnomon (0)

def gnomon (* on, ** kw) :

   """
   gnomon ()
      or gnomon (onoff)
     Toggle the gnomon display. If on is present and non-zero,
     turn on the gnomon. If zero, turn it off.

     The gnomon shows the X, Y, and Z axis directions in the
     object coordinate system. The directions are labeled.
     The gnomon is always infinitely far behind the object
     (away from the camera).

     There is a mirror-through-the-screen-plane ambiguity in the
     display which is resolved in two ways: (1) the (X, Y, Z)
     coordinate system is right-handed, and (2) If the tip of an
     axis projects into the screen, its label is drawn in opposite
     polarity to the other text in the screen.
   """

#    (ZCM 4/4/97) Add keyword argument chr to allow specification
#    of the axis labels.

   global _gnomon, chr
   old = _gnomon
   if len (on) == 0 :
      _gnomon = 1 - _gnomon
   elif (on [0]) :
      _gnomon = 1
   else :
      _gnomon = 0
   if old != _gnomon :
      draw3_trigger ()
   if kw.has_key ("chr") :
      chr = kw ["chr"]
   else :
      chr = ["X", "Y", "Z"]

def _gnomon_draw ( ) :
   global chr
   o = array ( [0., 0., 0.],  Float)
   x1 = array ( [1., 0., 0.],  Float)
   y1 = array ( [0., 1., 0.],  Float)
   z1 = array ( [0., 0., 1.],  Float)
   xyz1 = array (getrot3_ ( ), copy = 1)
   xyz2 = array([[o,x1],[o,y1],[o,z1]])
   s1 = shape ( xyz1 )
   s2 = shape ( xyz2 )
   xyz = zeros ( (s2 [1], s2 [0], s1 [1] ), Float)
   xyz [0, :, :] = dot (transpose (xyz1), xyz2 [:, 0, :])
   xyz [1, :, :] = dot (transpose (xyz1), xyz2 [:, 1, :])
   xyz = .0013 * _gnomon_scale * xyz
   x1 = xyz [0:2, 0, 0:3]
   y1 = xyz [0:2, 1, 0:3]
   z1 = xyz [1, 2, 0:3]
   x0 = x1 [0]
   x1 = x1 [1]
   y0 = y1 [0]
   y1 = y1 [1]
   wid = min (_gnomon_scale / 18., 6.)
   if ( wid < 0.5 ) : wid = 0.
   plsys (0)
   pldj (x0 + _gnomon_x, y0 + _gnomon_y, x1 + _gnomon_x, y1 + _gnomon_y,
         width = wid, type = 1, legend = "")
   plsys (1)

   # Compute point size of labels (1/3 of axis length)
   pts = [8, 10, 12, 14, 18, 24] [digitize (_gnomon_scale / 3.0,
         array ([9, 11, 13, 16, 21], Int))]

   if _gnomon_scale < 21.0 :
      x1 = x1 * 21. / _gnomon_scale
      y1 = y1 * 21. / _gnomon_scale
   # label positions: first find shortest axis
   xy = sqrt (x1 * x1 + y1 * y1)
   xysum = add.reduce (xy)
   i = argmin (xy)          # mnx (xy)
   jk = [ [1, 2], [2, 0], [0, 1]] [i]
   j = jk [0]
   k = jk [1]
   if xy [i] < 1.e-7 * xysum : # guarantee not exactly zero
      x1 [i] = -1.e-6 * (x1 [j] + x1 [k] )
      y1 [i] = -1.e-6 * (y1 [j] + y1 [k] )
      xy [i] = sqrt (x1 [i] * x1 [i] + y1 [i] * y1 [i])
   xyi = xy [i]
   # next find axis nearest to shortest
   if abs (x1 [j] * y1 [i] - y1 [j] * x1 [i]) * xy [k] > \
      abs (x1 [k] * y1 [i] - y1 [k] * x1 [i]) * xy [j] :
      jk = j
      j = k
      k = jk
   # furthest axis first--move perpendicular to nearest axis
   xk = - y1 [j]
   yk = x1 [j]
   xy = sqrt (xk * xk + yk * yk)
   xk = xk / xy
   yk = yk / xy
   if (xk * x1 [k] + yk * y1 [k] < 0.0 ) :
      xk = - xk
      yk = - yk
   # nearer axis next--move perpendicular to furthest axis
   xj = - y1 [k]
   yj = x1 [k]
   xy = sqrt (xj * xj + yj * yj)
   xj = xj / xy
   yj = yj / xy
   if (xj * x1[j] + yj * y1 [j] < 0.0 ) :
      xj = - xj
      yj = - yj
   # shortest axis last -- move perpendicular to nearer
   xi = - y1 [j]
   yi = x1 [j]
   xy = sqrt (xi * xi + yi * yi)
   xi = xi / xy
   yi = yi / xy
   if (xi *x1 [i] + yi * y1 [i] < 0.0) :
      xi = - xi
      yi = - yi

   # shortest axis label may need adjustment
   d = 0.0013 * pts
   if xyi < d :
      # just center it in correct quadrant
      jk = sign_ (xi * xj + yi * yj)
      yi = sign_ (xi * xk + yi * yk)
      xi = jk * xj + yi * xk
      yi = jk * yj + yi * yk
      jk = sqrt (xi * xi + yi * yi)
      xi = xi / jk
      yi = yi / jk
   x = zeros (3, Float)
   y = zeros (3, Float)
   x [i] = xi
   x [j] = xj
   x [k] = xk
   y [i] = yi
   y [j] = yj
   y [k] = yk
   x = x * d
   y = y * d
   x = x + x1 + _gnomon_x
   y = y + y1 + _gnomon_y
   try :
      dum = chr
   except :
      chr = ["X", "Y", "Z"]
   gnomon_text_ (chr [i], x [i], y [i], pts, z1 [i] < 1.e-6)
   gnomon_text_ (chr [j], x [j], y [j], pts, z1 [j] < 1.e-6)
   gnomon_text_ (chr [k], x [k], y [k], pts, z1 [k] < 1.e-6)

try :
   dummy = _gnomon_scale
except :
   _gnomon_scale = 30.       # axes lengths in points
try :
   dummy = _gnomon_x
except :
   _gnomon_x = 0.18          # gnomon origin in system 0 coordinates
try :
   dummy = _gnomon_y
except :
   _gnomon_y = 0.42

def gnomon_text_ (chr, x, y, pts, invert) :
   # pts = 8, 10, 12, 14, 18, or 24
   col = "fg"
   if invert :
      plsys (0)
      plg (array ( [y, y]), array ( [x, x]), type = 1, width = 2.2 * pts,
           color = col, marks = 0, legend = "")
      plsys (1)
      col = "bg"
   plt (chr, x, y, justify = "CH", color = col, height = pts,
        font = "helvetica", opaque = 0)

from movie import *

g_nframes = 30

def spin3 (nframes = 30, axis = array ([-1, 1, 0],  Float), tlimit = 60.,
   dtmin = 0.0, bracket_time = array ([2., 2.],  Float), lims = None,
   timing = 0, angle = 2. * pi) :

   """
   spin3 ( ) or spin3 (nframes) os spin3 (nframes, axis)
     Spin the current 3D display list about AXIS over NFRAMES.  Keywords
     tlimit= the total time allowed for the movie in seconds (default 60),
     dtmin= the minimum allowed interframe time in seconds (default 0.0),
     bracket_time= (as for movie function in movie.i), timing = 1 if
     you want timing measured and printed out, 0 if not.

     The default AXIS is [-1,1,0] and the default NFRAMES is 30.
   SEE ALSO: rot3
   """

   # Note on global variables (ZCM 2/21/97):
   # I see no better way of sharing these between spin3 and _spin3
   #   than making them global. Otherwise one would have to pass
   #   them to movie, which would then send them as arguments to
   #   _spin3. But because movie may call other routines, every one
   #   of them would have to have these values, necessary or not.
   #   So I have started their names with underscores; at least
   #   this makes them inaccessible outside this module.
   global _phi, _theta, _dtheta
   global _g_nframes
   _g_nframes = nframes
   _dtheta = angle / (nframes - 1)
   _theta = arccos (axis [2] / sqrt (axis [0] * axis [0] + axis [1] * axis [1] +
                   axis [2] * axis [2]))
   inc = axis [0] == axis [1] == 0
   _phi = arctan2 (axis [1], axis [0] + inc)
   orig = save3 ( )
   movie (_spin3, tlimit, dtmin, bracket_time, lims, timing = 0)
   restore3 (orig)

def _spin3 (i) :
   global _g_nframes
   global _phi, _theta, _dtheta
   if i >= _g_nframes:
      return 0
   rot3 (za = -_phi)
   rot3 (ya = -_theta, za = _dtheta)
   rot3 (ya = _theta, za = _phi)
   lims = draw3 ( )
   limits (lims [0], lims [1], lims [2], lims [3])
   return 1



from ga_util import *
import scipy.stats as stats
rv = stats
#import scipy.io.dumb_shelve
import string, pdb
import os, sys, string
import time, pprint, types,copy
import anydbm, dumbdbm
#import thread, sync
if sys.platform != 'win32': 
	import fcntl
	timer = time.clock	#clock behaves differently work on linux
else:
    timer = time.time
    
dberror = dumbdbm.error

def max_score(pop): return max(map(lambda x: x.score(),pop))

class galg:
	"""A basic genetic algorithm.  The genetic algorithm is responsible
	   for evolving a population of genomes.  While the population and
	   the genomes are in charge of defining most of the genetic operators
	   such as selection, scaling, mutation, and crossover, it is the
	   genetic algorithm class that orchestrates the evolution and calls
	   the operators in the correct order.  Most of the work is done
	   in the **step()** method.
	"""
	valid_settings = ['pop_size','p_replace',
				'p_cross', 'p_mutate','p_deviation',
				'gens','rand_seed','rand_alg','dbase','update_rate']
	output_settings = ['crossover','selector', 'scaler','genome_type']
	default_settings = {'pop_size':150,'p_replace':.8,
				'p_cross': .8, 'p_mutate':'gene',
				'p_deviation': 0.,'gens':35,
				'rand_seed':0,'rand_alg':'CMRG',
				'update_rate': 10000,'dbase':''}
	default_verbose = 1

	def __init__(self,pop):
		self.verbose = self.default_verbose
		self.settings = copy.copy(galg.default_settings)
		self.pop = pop
	def test_settings(self,settings):
		for key in settings.keys(): 
			try: 
				self.output_settings.index(key)
				print 'Warning: The key "%s" in settings is readonly.' % key
			except ValueError:	
				try: self.valid_settings.index(key)
				except ValueError: 
					print 'Warning: The key "%s" in not a valid setting.' % key
					print 'The valid settings are %s' % self.valid_settings
	
	def initialize(self,reseed = 1): 
		b = timer()
		self.test_settings(self.settings)
		self.gen = 0
		sd = self.settings['rand_seed']; alg = self.settings['rand_alg']
		if reseed: rv.initialize(seed = sd, algorithm = alg)
		self.settings['seed_used'] = rv.initial_seed()
		self._print('initializing... seed = %d' % self.settings['seed_used'])
		self.crossover = self.pop.model_genome.crossover # get the crossover op from the first genome
		self.pop.settings = self.settings #should these be shared?
		self.size_pop(self.settings['pop_size'])
			
		self.settings['crossover'] = string.split(str(self.crossover))[0][1:]
		self.settings['selector'] = string.split(str(self.pop.selector))[0][1:]
		self.settings['scaler'] = string.split(str(self.pop.scaler))[0][1:]
		self.settings['genome_type'] = string.split(str(self.pop.model_genome))[0][1:]
#		self._print(self.settings)
		
		self.pop.initialize(self.settings); 
		self.stats = {'selections':0,'crossovers':0,'mutations':0,
				  'replacements':0,'pop_evals':1,'ind_evals':0}
		self.stats.update(self.pop.stats)
		self.step_time = timer() - b
		self.init_dbase()
	def size_pop(self,s):
		self.settings['pop_size'] = s
		self.pop._size(s)

	def step(self,steps=1):
		sz = len(self.pop)
		replace = int(self.settings['p_replace'] * len(self.pop))
		p_crossover = self.settings['p_cross']
		for st in range(steps):
			b = timer()
			for i in range(0,replace,2):
				mom,dad= self.pop.select(2)
				self.stats['selections'] = self.stats['selections'] + 2
				if flip_coin(p_crossover): 
					try: 
						bro,sis = self.crossover((mom,dad))
						self.stats['crossovers'] = self.stats['crossovers'] + 2
						self.pop.append(bro); self.pop.append(sis)				
					except ValueError: 
						#crossover failed - just act as if this iteration never happened
						i = i - 2 
						#print 'crossover failure - ignoring and continuing'
				else: 
					self.pop.append(mom.clone());self.pop.append(dad.clone());
			if replace % 2: #we did one to many - remove the last individual
				del self.pop[-1]
				self.stats['crossovers'] = self.stats['crossovers'] - 1
			e1 = timer();
			self.stats['mutations'] = self.stats['mutations'] + self.pop[sz:].mutate()
#			for ind in self.pop[sz:]:
#				m = ind.mutate()
#				self.stats['mutations'] = self.stats['mutations'] + m
			e2 = timer();
			self.pop.touch()
			self.pop.evaluate()
			e3 = timer();
			del self.pop[sz:] #touch removed from del
			self.pop.scale()
			self.pop.update_stats()
			self.stats['pop_evals'] = self.stats['pop_evals'] + 1
			self.gen = self.gen + 1
			e = timer(); self.step_time = e - b
			#print 'cross:',e1-b,'mutate:',e2-e1,'eval:',e3-e2,'rest:',e-e3
		self.stats.update(self.pop.stats)	
		self.db_entry['best_scores'].append(self.stats['current']['max'])

	def evolve(self):
		b = timer()
		self.initialize()
		self.pre_evolve()
		self.p_dev = self.pop_deviation()
		self.iteration_output()
		while (	self.gen < self.settings['gens'] and
				self.settings['p_deviation'] < self.p_dev  ):
			self.step()
			self.p_dev = self.pop_deviation()
			self.iteration_output()
			if(self.gen % self.settings['update_rate'] == 0):
				self.update_dbase()
		self.update_dbase() #enter status prior to post_evolve in dbase
		self.post_evolve()
		self.db_entry['run_time'] = timer() - b
		self.write_dbase()
	def iteration_output(self):
		output = ( 'gen: ' + `self.gen` + ' ' 
		         + 'max: ' + `self.stats['current']['max']`  + ' ' 
		         + 'dev: ' + `self.p_dev` + ' ' 
		        + 'eval time: ' + `self.step_time` + ' ')
		self._print( output )
			
	def pre_evolve(self): 	pass	
	def post_evolve(self): 	pass	
	def pop_deviation(self):
		#calculate the std deviation across all populations as a percent of mean
		scores = self.pop.scores()
		denom = my_mean(scores)
		if denom == 0.: denom = .0001  # what should I do here?
		return abs(my_std(scores)/denom)
	#dbase stuff
	def init_dbase(self):
		self.db_entry = {}
		self.db_entry['settings'] = self.settings
		t=time.time()
		self.db_entry['raw_time'] = t
		self.db_entry['time'] = time.ctime(t)
		self.db_entry['best_scores'] = [self.stats['current']['max']]
		self.db_entry['stats'] = [copy.deepcopy(self.stats)]
		self.db_entry['step_time'] = [self.step_time]
		self.db_entry['optimization_type'] = string.split(str(self.__class__))[0][1:]
	
	def update_dbase(self):
#		self.db_entry['best_scores'].append(self.pop.best().score())
		self.db_entry['stats'].append(copy.deepcopy(self.stats))
		self.db_entry['step_time'].append(self.step_time)

	def write_dbase(self):	
		"""This does not do file locking on NT - which isn't that big
		   a deal because at the most, two runs are going at a time, and
		   they are unlikely going to write at the same time (but could).
		   On NT, hopefully we're using the gdbm module which does automatic
		   file locking.
		"""
		if(self.settings['dbase'] != ''):
			fname= self.settings['dbase']
			try: 
				if sys.platform == 'win32': pass
				else:
					f = open(fname +'.lock','a')
					fcntl.flock(f.fileno(),fcntl.LOCK_EX)
				try:
					try: db = my_shelve.open(fname,'w')
					except dberror: db = my_shelve.open(fname,'c')	
					keys = db.keys()
					if(len(keys) == 0): self.dbkey = `1`
					else:
						gkeys=[]
						for k in keys:
							try: gkeys.append(string.atoi(k))
							except ValueError: pass
						self.dbkey = `max(gkeys)+1`
					print 'DB NAME: ', self.settings['dbase'], 'KEY: ', self.dbkey
					db[self.dbkey] = self.db_entry 
					db.close()
				except: pass #if an error occured, we still need to unlock the db	
				if sys.platform == 'win32': pass
				else:
					fcntl.flock(f.fileno(),fcntl.LOCK_UN)
					f.close()
			except: 	
				if sys.platform == 'win32': pass
				else:
					f = open('error.lock','a')
					f.write(os.environ['HOST'])
					f.close()

		else:	"no dbase specified"

	def _print(self,val, level = 1):
		if(self.verbose >= level):
			if type(val) == types.StringType: print val
			else:
				pp = pprint.PrettyPrinter(indent=4)
				pp.pprint(val)
	
	
	ALL = -1
class m_galg(galg):
	valid_settings = galg.valid_settings + ['num_pops', 'migrants']
	default_settings = galg.default_settings
	default_settings['pop_size'] = 30; default_settings['num_pops'] = 4
	default_settings['migrants'] = 2
					  
	verbose = 1
	def __init__(self,pop):
		galg.__init__(self,pop)
#		self.GAs = self.GAs + [galg(pop.clone())]
		self.settings = copy.copy(self.default_settings)

	def initialize(self, mode = 'serial'): 
		b = timer()
		#same as galg
		self.test_settings(self.settings)
		self.gen = 0
		sd = self.settings['rand_seed']; alg = self.settings['rand_alg']
		rv.initialize(seed = sd, algorithm = alg)
		self.settings['seed_used'] = rv.initial_seed()
		self._print('initializing... seed = %d' % self.settings['seed_used'])
		self.crossover = self.pop[0].crossover # get the crossover op from the first genome
		self.pop.settings = self.settings
		#end same as galg
		
		#set up my population to hold the best from each sub-pop
		self.pop._size(0) #erase any current member of the pop
		self.pop._size(self.settings['num_pops'])
		self.crossover = self.pop[0].crossover
	
		#extract the galg settings so we don't get a ton of warnings
		#and create the sub ga_s
		sub_ga_settings = {}
		self.GAs = []
		for key in galg.valid_settings:
			sub_ga_settings[key] = self.settings[key]
		for i in range(self.settings['num_pops']): 
			self.GAs.append(galg(self.pop.clone()))
			self.GAs[i].settings = sub_ga_settings.copy()

		self.settings['crossover'] = string.split(str(self.crossover))[0][1:]
		self.settings['selector'] = string.split(str(self.pop.selector))[0][1:]
		self.settings['scaler'] = string.split(str(self.pop.scaler))[0][1:]
		self.settings['genome_type'] = string.split(str(self.pop.model_genome))[0][1:]
		self._print(self.settings)

		if mode[0] == 'p' or mode[0] == 'P':
		    """
			sys.setcheckinterval(1000)
			finished = sync.event()
			bar = sync.barrier(len(self.GAs))
			for ga in self.GAs: 
				thread.start_new_thread(GA_initializer,(bar,finished,ga))
			finished.wait()
			sys.setcheckinterval(10)
			"""
		else:
			for ga in self.GAs: ga.initialize(reseed = 0)								
		cnt = 0		
		for ga in self.GAs: 
			self.pop[cnt] = ga.pop.best()
			cnt = cnt + 1
		self.pop.sort() 			
		self.init_stats()
		self.step_time = timer() - b
		self.init_dbase()

	def init_stats(self):
		#first set up the pops stats, since we don't officially initialize it.
		self.pop.stats = {'current':{},'initial':{},'overall':{}}
		self.pop.stats['selections'] =0; self.pop.stats['crossovers'] =0
		self.pop.stats['mutations'] = 0; self.pop.stats['replacements'] = 0
		self.pop.stats['ind_evals'] = 0
		self.stats = self.pop.stats.copy()
		self.update_stats()
	def update_stats(self):
		"""Gather statistics from the various populations to the mga's population.
		"""
		sum_fields = ['selections','crossovers','mutations','replacements','ind_evals']
		s = []
		for ga in self.GAs:
			for field in sum_fields:
				self.pop.stats[field] = self.pop.stats[field] + ga.stats[field]
			s = s + ga.pop.scores().tolist()			

		self.pop.stats['current']['max'] = self.pop.best().score()
		self.pop.stats['current']['avg'] = my_mean(s)
		self.pop.stats['current']['min'] = min(s)
		if len(s) > 1: self.pop.stats['current']['dev'] = my_std(s)
		else: self.pop.stats['current']['dev'] = 0	
		try: self.pop.stats['overall']['max'] = max(self.pop.stats['overall']['max'],
								    self.pop.stats['current']['max'])
		except KeyError: self.pop.stats['overall']['max'] = self.pop.stats['current']['max']
		try: self.pop.stats['overall']['min'] = min(self.pop.stats['overall']['min'],
								    self.pop.stats['current']['min'])
		except KeyError: self.pop.stats['overall']['min'] = self.pop.stats['current']['min']
		self.pop.stats
		self.pop.stats['pop_evals'] = self.GAs[0].stats['pop_evals']
		self.stats.update(self.pop.stats)


	def step(self,steps=1,mode = 'serial'):
		for st in range(steps):
			b = timer()
			cnt = 0
			#self.pop._size(0) # used if we keep a single pop
			if mode[0] == 'p' or mode[0] == 'P':
			    """
				sys.setcheckinterval(100)
				finished = sync.event()
				bar = sync.barrier(len(self.GAs))
				for ga in self.GAs: 
					thread.start_new_thread(GA_stepper,(bar,finished,ga))
				finished.wait()
				sys.setcheckinterval(10)
				"""
			else:
				for ga in self.GAs: ga.step()								
 
			for ga in self.GAs: 
				#replace the worst member of the local pop 
				self.pop[-1] = ga.pop.best()			
				self.pop.sort()
				#probabaly not the fast approach to things, but... keeps an itelligent pop
				#for ind in ga.pop: self.pop.append(ind)	
	
			self.migrate()
			self.gen = self.gen + 1
			e = timer(); self.step_time = e - b
			self.update_stats()
			self.db_entry['best_scores'].append(self.stats['current']['max'])
			
	def pop_deviation(self):
		"""calculate the std deviation across all populations"""
		all_scores = []
		for ga in self.GAs:
			all_scores = all_scores + ga.pop.scores().tolist()
		if len(all_scores) > 1:
			denom = my_mean(all_scores)
			if denom == 0.: denom = .0001  # what should I do here?
			return abs(my_std(all_scores)/denom)
		return 0

	def evolve(self, mode = 'serial'):
		b = timer()
		self.initialize(mode)
		self.pre_evolve()
		self.p_dev = self.pop_deviation()
		self.iteration_output()
		while (	self.gen < self.settings['gens'] and
				self.settings['p_deviation'] < self.p_dev  ):
			self.step(1,mode)
			self.p_dev = self.pop_deviation()
			self.iteration_output()
		self.update_dbase() #enter status prior to post_evolve in dbase
		self.post_evolve()
		self.db_entry['run_time'] = timer() - b
		self.write_dbase()

	def migrate(self):
		"""Migration moves members from one population to another.  It takes
		   the best N individuals of GAs[0] and puts clones of them into 
		   GAs[1], replacing the worst individuals.  Likewise,
		   GAs[1] best replace GAs[2] worst.  GAs[-1] best are moved
		   to GAs[0].  This 'stepping stone' migration of individuals allows
		   good ideas to move from one population to another, but still
		   allows the individual population to maintain som diversity.
		"""   
		if len(self.GAs) == 1: return
		migrants = self.settings['migrants']
		if migrants > self.settings['pop_size']:
			migrants = self.settings['pop_size']

		movers = []
		for i in range(migrants):
			movers.append(self.GAs[0].pop[i])
		for ga in self.GAs[1:]:	
			for i in range(migrants):
				ga.pop[-i] = movers[i].clone() #replace the worst individual
				movers[i] = ga.pop[i]
		for i in range(migrants):
			self.GAs[0].pop[-i] = movers[i].clone() #replace the worst individual


"""
def GA_stepper(bar,finished,GA):
	t1 = timer()
	GA.step()
	t2 = timer()
	print 'thread ' + `thread.get_ident()` + 'time ' + `t2-t1` + ' sec.'
	bar.enter()
	finished.post()

def GA_initializer(bar,finished,GA):
	t1 = timer()
	GA.initialize(reseed = 0)
	t2 = timer()
	print 'thread ' + `thread.get_ident()` + 'time ' + `t2-t1` + ' sec.'
	bar.enter()
	finished.post()			
"""	




from info_scipy_base import __doc__
from scipy_base_version import scipy_base_version as __version__

from ppimport import ppimport, ppimport_attr

# The following statement is equivalent to
#
#   from Matrix import Matrix as mat
#
# but avoids expensive LinearAlgebra import when
# Matrix is not used.
mat = ppimport_attr(ppimport('Matrix'), 'Matrix')

# Force Numeric to use scipy_base.fastumath instead of Numeric.umath.
import fastumath  # no need to use scipy_base.fastumath
import sys as _sys
_sys.modules['umath'] = fastumath

import Numeric
from Numeric import *

import limits
from type_check import *
from index_tricks import *
from function_base import *
from shape_base import *
from matrix_base import *



from polynomial import *
from scimath import *
from machar import *
from pexec import *

if Numeric.__version__ < '23.5':
    matrixmultiply=dot

Inf = inf = fastumath.PINF
try:
    NAN = NaN = nan = fastumath.NAN
except AttributeError:
    NaN = NAN = nan = fastumath.PINF/fastumath.PINF

from scipy_test.testing import ScipyTest
test = ScipyTest('scipy_base').test

if _sys.modules.has_key('scipy_base.Matrix') \
   and _sys.modules['scipy_base.Matrix'] is None:
    del _sys.modules['scipy_base.Matrix']




# Author:  Travis Oliphant, 2002
#

from __future__ import nested_scopes

import math
import statlib
import stats
import distributions
import inspect
from scipy_base import isscalar, r_, log, sum, around, unique, asarray
from scipy_base import zeros, arange, sort, amin, amax, any, where, \
     array, atleast_1d, sqrt, ceil, floor, array, poly1d, compress, not_equal, \
     pi, exp, ravel
import scipy
import types
import scipy.optimize as optimize
import futil
import scipy_base as sb


def find_repeats(arr):
    """Find repeats in arr and return (repeats, repeat_count)
    """    
    v1,v2, n = futil.dfreps(arr)
    return v1[:n],v2[:n]


##########################################################
###  Bayesian confidence intervals for mean, variance, std
##########################################################

##  Assumes all is known is that mean, and std (variance) exist
##   and are the same for all the data.  Uses Jeffrey's prior
##
##  Returns alpha confidence interval for the mean, variance,
##      and std.

def bayes_mvs(data,alpha=0.90):
    """Return Bayesian confidence intervals for the mean, var, and std.

    Assumes 1-d data all has same mean and variance and uses Jeffrey's prior
    for variance and std.

    alpha gives the probability that the returned interval contains
    the true parameter.

    Uses peak of conditional pdf as starting center.

    Returns (peak, (a, b)) for each of mean, variance and standard deviation.
    """
    x = ravel(data)
    n = len(x)
    assert(n > 1)
    n = float(n)
    xbar = sb.add.reduce(x)/n
    C = sb.add.reduce(x*x)/n - xbar*xbar
    #
    fac = sqrt(C/(n-1))
    tval = distributions.t.ppf((1+alpha)/2.0,n-1)
    delta = fac*tval
    ma = xbar - delta
    mb = xbar + delta
    mp = xbar
    #
    fac = n*C/2.0
    peak = 2/(n+1.)
    a = (n-1)/2.0    
    F_peak = distributions.invgamma.cdf(peak,a)
    q1 = F_peak - alpha/2.0
    q2 = F_peak + alpha/2.0
    if (q1 < 0):  # non-symmetric area
        q2 = alpha
        va = 0.0
    else:
        va = fac*distributions.invgamma.ppf(q1,a)
    vb = fac*distributions.invgamma.ppf(q2,a)
    vp = peak*fac
    #
    fac = sqrt(fac)
    peak = sqrt(2./n)
    F_peak = distributions.gengamma.cdf(peak,a,-2)
    q1 = F_peak - alpha/2.0
    q2 = F_peak + alpha/2.0
    if (q1 < 0):
        q2 = alpha
        sta = 0.0
    else:
        sta = fac*distributions.gengamma.ppf(q1,a,-2)
    stb = fac*distributions.gengamma.ppf(q2,a,-2)
    stp = peak*fac
        
    return (mp,(ma,mb)),(vp,(va,vb)),(stp,(sta,stb))
    


################################
##  K-Statistic
################################

###
## The n-th k-statistic is the unique symmetric unbiased estimator of
##   the n-th cumulant kappa_n
## The cumulants are related to central moments but are specifically defined
##   using a power series expansion of the logarithm of the characteristic
##   function (which is the Fourier transform of the PDF).
##   In particular let phi(t) be the characteristic function, then
##                     _
##         ln phi(t) = > kappa_n (it)^n / n!    (sum from n=0 to inf)
##                     -
##
##   The first few cumulants (kappa_n)  in terms of central moments (mu_n) are
##    kappa_1 = mu_1
##    kappa_2 = mu_2
##    kappa_3 = mu_3
##    kappa_4 = mu_4 - 3*mu_2**2
##    kappa_5 = mu_5 - 10*mu_2 * mu_3
##
## Source:  http://mathworld.wolfram.com/Cumulant.html
##          http://mathworld.wolfram.com/k-Statistic.html
##

def kstat(data,n=2):
    """Return the nth k-statistic (1<=n<=4 so far).

    The nth k-statistic is the unique symmetric unbiased estimator of the nth
    cumulant kappa_n
    """
    if n>4 or n<1:
        raise ValueError, "k-statistics only supported for 1<=n<=4"
    n = int(n)
    S = zeros(n+1,'d')
    data = ravel(data)
    N = len(data)
    for k in range(1,n+1):
        S[k] = sum(data**k)
    if n==1:
        return S[1]*1.0/N
    elif n==2:
        return (N*S[2]-S[1]**2.0)/(N*(N-1.0))
    elif n==3:
        return (2*S[1]**3 - 3*N*S[1]*S[2]+N*N*S[3]) / (N*(N-1.0)*(N-2.0))
    elif n==4:
        return (-6*S[1]**4 + 12*N*S[1]**2 * S[2] - 3*N*(N-1.0)*S[2]**2 - \
                4*N*(N+1)*S[1]*S[3] + N*N*(N+1)*S[4]) / \
                (N*(N-1.0)*(N-2.0)*(N-3.0))
    else:
        raise ValueError, "Should not be here."

def kstatvar(data,n=2):
    """Returns an unbiased estimator of the variance of the k-statistic:  n=1 or 2
    """
    data = ravel(data)
    N = len(data)
    if n==1:
        return kstat(data,n=2)*1.0/N
    elif n==2:
        k2 = kstat(data,n=2)
        k4 = kstat(data,n=4)
        return (2*k2*k2*N + (N-1)*k4)/(N*(N+1))
    else:
        raise ValueError, "Only n=1 or n=2 supported."


#__all__ = ['probplot','ppcc_max','ppcc_plot','boxcox','boxcox_llf',
#           'boxcox_normplot','boxcox_normmax','shapiro']

def probplot(x, sparams=(), dist='norm', fit=1, plot=None):
    """Return (osm, osr){,(scale,loc,r)} where (osm, osr) are order statistic
    medians and ordered response data respectively so that plot(osm, osr)
    is a probability plot.  If fit==1, then do a regression fit and compute the
    slope (scale), intercept (loc), and correlation coefficient (r), of the
    best straight line through the points.  If fit==0, only (osm, osr) is
    returned.
    
    sparams is a tuple of shape parameter arguments for the distribution.
    """
    N = len(x)
    Ui = zeros(N)*1.0
    Ui[-1] = 0.5**(1.0/N)
    Ui[0] = 1-Ui[-1]
    i = arange(2,N)
    Ui[1:-1] = (i-0.3175)/(N+0.365)
    try:
        ppf_func = eval('distributions.%s.ppf'%dist)
    except AttributError:
        raise dist, "is not a valid distribution with a ppf."
    if sparams is None:
        sparams = ()
    if isscalar(sparams):
        sparams = (sparams,)
    if not isinstance(sparams,types.TupleType):
        sparams = tuple(sparams)
    res = inspect.getargspec(ppf_func)
    if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
            0.0==res[-1][-2] and 1.0==res[-1][-1]):
        raise ValueError, "Function has does not have default location", \
              "and scale parameters\n  that are 0.0 and 1.0 respectively."
    if (len(sparams) < len(res[0])-len(res[-1])-1) or \
       (len(sparams) > len(res[0])-3):
        raise ValueError, "Incorrect number of shape parameters."
    osm = ppf_func(Ui,*sparams)
    osr = sort(x)
    if fit or (plot is not None):
        # perform a linear fit.
        slope, intercept, r, prob, sterrest = stats.linregress(osm,osr)
    if plot is not None:
        try:
            import scipy.xplt as xplt
            xplt.limits()
        except: pass
        plot.plot(osm, osr, 'o', osm, slope*osm + intercept)
        plot.title('Probability Plot')
        plot.xlabel('Order Statistic Medians')
        plot.ylabel('Ordered Values')
        try: plot.expand_limits(5)
        except: pass
        xmin,xmax= amin(osm),amax(osm)
        ymin,ymax= amin(x),amax(x)
        pos = xmin+0.70*(xmax-xmin), ymin+0.01*(ymax-ymin)
        try: plot.addtext("r^2^=%1.4f" % r, xy=pos,tosys=1)
        except: pass
    if fit:
        return (osm, osr), (slope, intercept, r)
    else:
        return osm, osr

def ppcc_max(x, brack=(0.0,1.0), dist='tukeylambda'):
    """Returns the shape parameter that maximizes the probability plot
    correlation coefficient for the given data to a one-parameter
    family of distributions.

    See also ppcc_plot
    """
    try:
        ppf_func = eval('distributions.%s.ppf'%dist)
    except AttributError:
        raise dist, "is not a valid distribution with a ppf."
    res = inspect.getargspec(ppf_func)
    if not ('loc' == res[0][-2] and 'scale' == res[0][-1] and \
            0.0==res[-1][-2] and 1.0==res[-1][-1]):
        raise ValueError, "Function has does not have default location", \
              "and scale parameters\n  that are 0.0 and 1.0 respectively."
    if (1 < len(res[0])-len(res[-1])-1) or \
       (1 > len(res[0])-3):
        raise ValueError, "Must be a one-parameter family."
    N = len(x)
    # compute uniform median statistics
    Ui = zeros(N)*1.0
    Ui[-1] = 0.5**(1.0/N)
    Ui[0] = 1-Ui[-1]
    i = arange(2,N)
    Ui[1:-1] = (i-0.3175)/(N+0.365)
    osr = sort(x)
    # this function computes the x-axis values of the probability plot
    #  and computes a linear regression (including the correlation)
    #  and returns 1-r so that a minimization function maximizes the
    #  correlation
    def tempfunc(shape, mi, yvals, func):
        xvals = func(mi, shape)
        r, prob = stats.pearsonr(xvals, yvals)
        return 1-r
    return optimize.brent(tempfunc, brack=brack, args=(Ui, osr, ppf_func))

def ppcc_plot(x,a,b,dist='tukeylambda', plot=None, N=80):
    """Returns (shape, ppcc), and optionally plots shape vs. ppcc
    (probability plot correlation coefficient) as a function of shape
    parameter for a one-parameter family of distributions from shape
    value a to b.

    See also ppcc_max
    """
    svals = r_[a:b:complex(N)]
    ppcc = svals*0.0
    k=0
    for sval in svals:
        r1,r2 = probplot(x,sval,dist=dist,fit=1)
        ppcc[k] = r2[-1]
        k += 1
    if plot is not None:
        try:
            import scipy.xplt as xplt
            xplt.limits()
        except: pass
        plot.plot(svals, ppcc, 'x')
        plot.title('(%s) PPCC Plot' % dist)
        plot.xlabel('Prob Plot Corr. Coef.',deltay=-0.01)
        plot.ylabel('Shape Values',deltax=-0.01)
        try: plot.expand_limits(5)
        except: pass
    return svals, ppcc

def boxcox_llf(lmb, data):
    """The boxcox log-likelihood function.
    """
    N = len(data)
    y = boxcox(data,lmb)
    my = stats.mean(y)
    f = (lmb-1)*sum(log(data))
    f -= N/2.0*log(sum((y-my)**2.0/N))
    return f

def _boxcox_conf_interval(x, lmax, alpha):
    # Need to find the lambda for which
    #  f(x,lmbda) >= f(x,lmax) - 0.5*chi^2_alpha;1
    fac = 0.5*distributions.chi2.ppf(1-alpha,1)
    target = boxcox_llf(lmax,x)-fac
    def rootfunc(lmbda,data,target):
        return boxcox_llf(lmbda,data) - target
    # Find positive endpont
    newlm = lmax+0.5
    N = 0
    while (rootfunc(newlm,x,target) > 0.0) and (N < 500):
        newlm += 0.1
        N +=1
    if (N==500):
        raise RuntimeError, "Could not find endpoint."
    lmplus = optimize.brentq(rootfunc,lmax,newlm,args=(x,target))
    newlm = lmax-0.5
    N = 0
    while (rootfunc(newlm,x,target) > 0.0) and (N < 500):
        newlm += 0.1
        N +=1
    if (N==500):
        raise RuntimeError, "Could not find endpoint."
    lmminus = optimize.brentq(rootfunc,newlm,lmax,args=(x,target))
    return lmminus,lmplus
       
def boxcox(x,lmbda=None,alpha=None):
    """Return a positive dataset tranformed by a Box-Cox power transformation.

    If lmbda is not None, do the transformation for that value.

    If lmbda is None, find the lambda that maximizes the log-likelihood
    function and return it as the second output argument.

    If alpha is not None, return the 100(1-alpha)% confidence interval for
    lambda as the third output argument. 
    """
    if any(x < 0):
        raise ValueError, "Data must be positive."
    if lmbda is not None:  # single transformation
        lmbda = lmbda*(x==x)
        y = where(lmbda == 0, log(x), (x**lmbda - 1)/lmbda)
        return y
    # Otherwise find the lmbda that maximizes the log-likelihood function.
    def tempfunc(lmb, data):  # function to minimize
        return -boxcox_llf(lmb,data)
    lmax = optimize.brent(tempfunc, brack=(-2.0,2.0),args=(x,))
    y, lmax = boxcox(x, lmax)
    if alpha is None:
        return y, lmax
    # Otherwise find confidence interval
    interval = _boxcox_conf_interval(x, lmax, alpha)
    return y, lmax, interval

    
def boxcox_normmax(x,brack=(-1.0,1.0)):
    N = len(x)
    # compute uniform median statistics
    Ui = zeros(N)*1.0
    Ui[-1] = 0.5**(1.0/N)
    Ui[0] = 1-Ui[-1]
    i = arange(2,N)
    Ui[1:-1] = (i-0.3175)/(N+0.365)
    # this function computes the x-axis values of the probability plot
    #  and computes a linear regression (including the correlation)
    #  and returns 1-r so that a minimization function maximizes the
    #  correlation
    xvals = distributions.norm.ppf(Ui)
    def tempfunc(lmbda, xvals, samps):
        y = boxcox(samps,lmbda)
        yvals = sort(y)
        r, prob  = stats.pearsonr(xvals, yvals)
        return 1-r
    return optimize.brent(tempfunc, brack=brack, args=(xvals, x))
        

def boxcox_normplot(x,la,lb,plot=None,N=80):
    svals = r_[la:lb:complex(N)]
    ppcc = svals*0.0
    k = 0
    for sval in svals:
        r1,r2 = probplot(x,dist='norm',fit=1)
        ppcc[k] = r2[-1]
        k +=1
    if plot is not None:
        try:
            import scipy.xplt as xplt
            xplt.limits()
        except: pass
        plot.plot(svals, ppcc, 'x')
        plot.title('Box-Cox Normality Plot')
        plot.xlabel('Prob Plot Corr. Coef.',deltay=-0.01)
        plot.ylabel('Transformation parameter',deltax=-0.01)
        try: plot.expand_limits(5)
        except: pass
    return svals, ppcc

def shapiro(x,a=None,reta=0):
    """Shapiro and Wilk test for normality.

    Given random variates x, compute the W statistic and its p-value
    for a normality test.

    If p-value is high, one cannot reject the null hypothesis of normality
    with this test.  P-value is probability that the W statistic is
    as low as it is if the samples are actually from a normal distribution.

    Output:  W statistic and its p-value

              if reta is nonzero then also return the computed "a" values
                 as the third output.  If these are known for a given size
                 they can be given as input instead of computed internally. 
    
    """
    N = len(x)
    if N < 3:
        raise ValueError, "Data must be at least length 3."
    if a is None:
        a = zeros(N,'f')
        init = 0
    else:
        assert(len(a) == N/2), "a must be == len(x)/2"
        init = 1
    y = sort(x)
    a,w,pw,ifault = statlib.swilk(y,a[:N/2],init)
    if not ifault in [0,2]:
        print ifault
    if N > 5000:
        print "p-value may not be accurate for N > 5000."
    if reta:
        return w, pw, a
    else:        
        return w, pw

# Values from Stephens, M A, "EDF Statistics for Goodness of Fit and
#             Some Comparisons", Journal of he American Statistical
#             Association, Vol. 69, Issue 347, Sept. 1974, pp 730-737
_Avals_norm = array([0.576, 0.656, 0.787, 0.918, 1.092])
_Avals_expon  = array([0.922, 1.078, 1.341, 1.606, 1.957])
# From Stephens, M A, "Goodness of Fit for the Extreme Value Distribution",
#             Biometrika, Vol. 64, Issue 3, Dec. 1977, pp 583-588.
_Avals_gumbel = array([0.474, 0.637, 0.757, 0.877, 1.038])
# From Stephens, M A, "Tests of Fit for the Logistic Distribution Based
#             on the Empirical Distribution Function.", Biometrika, 
#             Vol. 66, Issue 3, Dec. 1979, pp 591-595.
_Avals_logistic = array([0.426, 0.563, 0.660, 0.769, 0.906, 1.010])
def anderson(x,dist='norm'):
    """Anderson and Darling test for normal, exponential, or Gumbel
    (Extreme Value Type I) distribution.

    Given samples x, return A2, the Anderson-Darling statistic,
    the significance levels in percentages, and the corresponding
    critical values.

    Critical values provided are for the following significance levels
    norm/expon:   15%, 10%, 5%, 2.5%, 1%
    Gumbel:       25%, 10%, 5%, 2.5%, 1%
    logistic:     25%, 10%, 5%, 2.5%, 1%, 0.5%

    If A2 is larger than these critical values then for that significance
    level, the hypothesis that the data come from a normal (exponential)
    can be rejected.
    """
    if not dist in ['norm','expon','gumbel','extreme1','logistic']:
        raise ValueError, "Invalid distribution."
    y = sort(x)
    xbar = stats.mean(x)
    N = len(y)
    if dist == 'norm':
        s = stats.std(x)
        w = (y-xbar)/s
        z = distributions.norm.cdf(w)
        sig = array([15,10,5,2.5,1])
        critical = around(_Avals_norm / (1.0 + 4.0/N - 25.0/N/N),3)
    elif dist == 'expon':
        w = y / xbar
        z = distributions.expon.cdf(w)
        sig = array([15,10,5,2.5,1])
        critical = around(_Avals_expon / (1.0 + 0.6/N),3)
    elif dist == 'logistic':
        def rootfunc(ab,xj,N):
            a,b = ab
            tmp = (xj-a)/b
            tmp2 = exp(tmp)
            val = [sum(1.0/(1+tmp2))-0.5*N,
                   sum(tmp*(1.0-tmp2)/(1+tmp2))+N]
            return array(val)
        sol0=array([xbar,stats.std(x)])
        sol = optimize.fsolve(rootfunc,sol0,args=(x,N),xtol=1e-5)
        w = (y-sol[0])/sol[1]
        z = distributions.logistic.cdf(w)
        sig = array([25,10,5,2.5,1,0.5])
        critical = around(_Avals_logistic / (1.0+0.25/N),3)
    else:
        def fixedsolve(th,xj,N):
            val = stats.sum(xj)*1.0/N
            tmp = exp(-xj/th)
            term = sum(xj*tmp)
            term /= sum(tmp)
            return val - term
        s = optimize.fixed_point(fixedsolve, 1.0, args=(x,N),xtol=1e-5)
        xbar = -s*log(sum(exp(-x/s))*1.0/N)
        w = (y-xbar)/s
        z = distributions.gumbel_l.cdf(w)
        sig = array([25,10,5,2.5,1])
        critical = around(_Avals_gumbel / (1.0 + 0.2/sqrt(N)),3)
    i = arange(1,N+1)
    S = sum((2*i-1.0)/N*(log(z)+log(1-z[::-1])))
    A2 = -N-S
    return A2, critical, sig


def _find_repeats(arr):
    """Find repeats in the array and return (repeats, repeat_count)
    """
    arr = sort(arr)
    lastval = arr[0]
    howmany = 0
    ind = 1
    N = len(arr)
    repeat = 0
    replist = []
    repnum = []
    while ind < N:
        if arr[ind] != lastval:
            if repeat:
                repnum.append(howmany+1)
                repeat = 0
                howmany = 0
        else:
            howmany += 1
            repeat = 1
            if (howmany == 1):
                replist.append(arr[ind])
        lastval = arr[ind]
        ind += 1
    if repeat:
        repnum.append(howmany+1)
    return replist, repnum

def ansari(x,y):
    """Determine if the scale parameter for two distributions with equal
    medians is the same using the Ansari-Bradley statistic.

    Specifically, compute the AB statistic and the probability of error
    that the null hypothesis is true but rejected with the computed
    statistic as the critical value.

    One can reject the null hypothesis that the ratio of variances is 1 if
    returned probability of error is small (say < 0.05)
    """
    x,y = asarray(x),asarray(y)
    n = len(x)
    m = len(y)
    if (m < 1):
        raise ValueError, "Not enough other observations."
    if (n < 1):
        raise ValueError, "Not enough test observations."
    N = m+n
    xy = r_[x,y]  # combine
    rank = stats.rankdata(xy)
    symrank = amin(array((rank,N-rank+1)),0)
    AB = sum(symrank[:n])
    uxy = unique(xy)
    repeats = (len(uxy) != len(xy))
    exact = ((m<55) and (n<55) and not repeats)
    if repeats and ((m < 55)  or (n < 55)):
        print "Ties preclude use of exact statistic."
    if exact:
        astart, a1, ifault = statlib.gscale(n,m)
        ind = AB-astart
        total = sum(a1)
        if ind < len(a1)/2.0:
            cind = int(ceil(ind))
            if (ind == cind):
                pval = 2.0*sum(a1[:cind+1])/total
            else:
                pval = 2.0*sum(a1[:cind])/total
        else:
            find = int(floor(ind))
            if (ind == floor(ind)):
                pval = 2.0*sum(a1[find:])/total
            else:
                pval = 2.0*sum(a1[find+1:])/total
        return AB, min(1.0,pval)
    
    # otherwise compute normal approximation
    if N % 2:  # N odd
        mnAB = n*(N+1.0)**2 / 4.0 / N
        varAB = n*m*(N+1.0)*(3+N**2)/(48.0*N**2)
    else:
        mnAB = n*(N+2.0)/4.0
        varAB = m*n*(N+2)*(N-2.0)/48/(N-1.0)
    if repeats:   # adjust variance estimates
        # compute sum(tj * rj**2)
        fac = sum(symrank**2)
        if N % 2: # N odd
            varAB = m*n*(16*N*fac-(N+1)**4)/(16.0 * N**2 * (N-1))
        else:  # N even
            varAB = m*n*(16*fac-N*(N+2)**2)/(16.0 * N * (N-1))
    z = (AB - mnAB)/sqrt(varAB)
    pval = (1-distributions.norm.cdf(abs(z)))*2.0
    return AB, pval

def bartlett(*args):
    """Perform Bartlett test with the null hypothesis that all input samples
    have equal variances.

    Inputs are sample vectors:  bartlett(x,y,z,...) 

    Outputs: (T, pval)

         T    -- the Test statistic
         pval -- significance level if null is rejected with this value of T
                 (prob. that null is true but rejected with this p-value.)  

    Sensitive to departures from normality.  The Levene test is
    an alternative that is less sensitive to departures from
    normality.

    References:

      http://www.itl.nist.gov/div898/handbook/eda/section3/eda357.htm

      Snedecor, George W. and Cochran, William G. (1989), Statistical
        Methods, Eighth Edition, Iowa State University Press.     
    """
    k = len(args)
    if k < 2:
        raise ValueError, "Must enter at least two input sample vectors."
    Ni = zeros(k)
    ssq = zeros(k,'d')
    for j in range(k):
        Ni[j] = len(args[j])
        ssq[j] = stats.var(args[j])
    Ntot = sum(Ni)
    spsq = sum((Ni-1)*ssq)/(1.0*(Ntot-k))
    numer = (Ntot*1.0-k)*log(spsq) - sum((Ni-1.0)*log(ssq))
    denom = 1.0 + (1.0/(3*(k-1)))*((sum(1.0/(Ni-1.0)))-1.0/(Ntot-k))
    T = numer / denom
    pval = distributions.chi2.sf(T,k-1) # 1 - cdf
    return T, pval


def levene(*args,**kwds):
    """Perform Levene test with the null hypothesis that all input samples
    have equal variances.

    Inputs are sample vectors:  bartlett(x,y,z,...)

    One keyword input, center, can be used with values
        center = 'mean', center='median' (default), center='trimmed'

    center='median' is recommended for skewed (non-normal) distributions
    center='mean' is recommended for symmetric, moderate-tailed, dist.
    center='trimmed' is recommended for heavy-tailed distributions.

    Outputs: (W, pval)

         W    -- the Test statistic
         pval -- significance level if null is rejected with this value of W
                 (prob. that null is true but rejected with this p-value.)  

    References:

       http://www.itl.nist.gov/div898/handbook/eda/section3/eda35a.htm

       Levene, H. (1960). In Contributions to Probability and Statistics:
         Essays in Honor of Harold Hotelling, I. Olkin et al. eds.,
         Stanford University Press, pp. 278-292. 
       Brown, M. B. and Forsythe, A. B. (1974), Journal of the American
         Statistical Association, 69, 364-367         
    """
    k = len(args)
    if k < 2:
        raise ValueError, "Must enter at least two input sample vectors."
    Ni = zeros(k)
    Yci = zeros(k,'d')
    if 'center' in kwds.keys():
        center = kwds['center']
    else:
        center = 'median'
    if not center in ['mean','median','trimmed']:
        raise ValueError, "Keyword argument <center> must be 'mean', 'median'"\
              + "or 'trimmed'."
    if center == 'median':
        func = stats.median
    elif center == 'mean':
        func = stats.mean
    else:
        func = stats.trim_mean
    for j in range(k):
        Ni[j] = len(args[j])
        Yci[j] = func(args[j])    
    Ntot = sum(Ni)

    # compute Zij's
    Zij = [None]*k
    for i in range(k):
        Zij[i] = abs(asarray(args[i])-Yci[i])
    # compute Zbari
    Zbari = zeros(k,'d')
    Zbar = 0.0
    for i in range(k):
        Zbari[i] = stats.mean(Zij[i])
        Zbar += Zbari[i]*Ni[i]
    Zbar /= Ntot

    numer = (Ntot-k)*sum(Ni*(Zbari-Zbar)**2)

    # compute denom_variance
    dvar = 0.0
    for i in range(k):
        dvar += sum((Zij[i]-Zbari[i])**2)

    denom = (k-1.0)*dvar

    W = numer / denom
    pval = distributions.f.sf(W,k-1,Ntot-k) # 1 - cdf
    return W, pval

def binom_test(x,n=None,p=0.5):
    """An exact (two-sided) test of the null hypothesis that the
    probability of success in a Bernoulli experiment is p.

    Inputs:

       x -- Number of successes (or a vector of length 2 giving the
              number of successes and number of failures respectively)
       n -- Number of trials (ignored if x has length 2)
       p -- Hypothesized probability of success

    Returns pval -- Probability that null test is rejected for this set
                    of x and n even though it is true.
    """
    x = atleast_1d(x)
    if len(x) == 2:
        n = x[1]+x[0]
        x = x[0]
    elif len(x) == 1:
        x = x[0]
        if n is None or n < x:
            raise ValueError, "n must be >= x"
    else:
        raise ValueError, "Incorrect length for x."

    if (p > 1.0) or (p < 0.0):
        raise ValueError, "p must be in range [0,1]"

    d = distributions.binom.pmf(x,n,p)
    rerr = 1+1e-7
    if (x*1.0/n < p):
        i = arange(x+1,n+1)
        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr)
        pval = distributions.binom.cdf(x,n,p) + distributions.binom.sf(n-y,n,p)
    else:
        i = arange(0,x)
        y = sum(distributions.binom.pmf(i,n,p) <= d*rerr)
        pval = distributions.binom.cdf(y-1,n,p) + distributions.binom.sf(x-1,n,p)

    return min(1.0,pval)

def _apply_func(x,g,func):
    # g is list of indices into x
    #  separating x into different groups
    #  func should be applied over the groups
    g = unique(r_[0,g,len(x)])
    output = []
    for k in range(len(g)-1):
        output.append(func(x[g[k]:g[k+1]]))
    return asarray(output)

def fligner(*args,**kwds):
    """Perform Levene test with the null hypothesis that all input samples
    have equal variances.

    Inputs are sample vectors:  bartlett(x,y,z,...)

    One keyword input, center, can be used with values
        center = 'mean', center='median' (default), center='trimmed'

    Outputs: (Xsq, pval)

         Xsq  -- the Test statistic
         pval -- significance level if null is rejected with this value of X
                 (prob. that null is true but rejected with this p-value.)  

    References:

       http://www.stat.psu.edu/~bgl/center/tr/TR993.ps

       Fligner, M.A. and Killeen, T.J. (1976). Distribution-free two-sample
       tests for scale. 'Journal of the American Statistical Association.'
       71(353), 210-213.
    """
    k = len(args)
    if k < 2:
        raise ValueError, "Must enter at least two input sample vectors."
    if 'center' in kwds.keys():
        center = kwds['center']
    else:
        center = 'median'
    if not center in ['mean','median','trimmed']:
        raise ValueError, "Keyword argument <center> must be 'mean', 'median'"\
              + "or 'trimmed'."
    if center == 'median':
        func = stats.median
    elif center == 'mean':
        func = stats.mean
    else:
        func = stats.trim_mean

    Ni = asarray([len(args[j]) for j in range(k)])
    Yci = asarray([func(args[j]) for j in range(k)])
    Ntot = sum(Ni)
    # compute Zij's
    Zij = [abs(asarray(args[i])-Yci[i]) for i in range(k)]
    allZij = []
    g = [0]
    for i in range(k):
        allZij.extend(list(Zij[i]))
        g.append(len(allZij))

    a = distributions.norm.ppf(stats.rankdata(allZij)/(2*(Ntot+1.0)) + 0.5)

    # compute Aibar
    Aibar = _apply_func(a,g,sum) / Ni
    anbar = stats.mean(a)
    varsq = stats.var(a)

    Xsq = sum(Ni*(asarray(Aibar)-anbar)**2.0)/varsq

    pval = distributions.chi2.sf(Xsq,k-1) # 1 - cdf
    return Xsq, pval


def mood(x,y):
    """Determine if the scale parameter for two distributions with equal
    medians is the same using a Mood test.

    Specifically, compute the z statistic and the probability of error
    that the null hypothesis is true but rejected with the computed
    statistic as the critical value.

    One can reject the null hypothesis that the ratio of scale parameters is
    1 if the returned probability of error is small (say < 0.05)
    """
    n = len(x)
    m = len(y)
    xy = r_[x,y]
    N = m+n
    if (N < 3):
        raise ValueError, "Not enough observations."
    ranks = stats.rankdata(xy)
    Ri = ranks[:n]
    M = sum((Ri - (N+1.0)/2)**2)
    # Approx stat.
    mnM = n*(N*N-1.0)/12
    varM = m*n*(N+1.0)*(N+2)*(N-2)/180
    z = (M-mnM)/sqrt(varM)
    p = distributions.norm.cdf(z)
    pval = 2*min(p,1-p)
    return z, pval


def oneway(*args,**kwds):
    """Test for equal means in two or more samples from the
    normal distribution.

    If the keyword parameter <equal_var> is true then the variances
    are assumed to be equal, otherwise they are not assumed to
    be equal (default).

    Return test statistic and the p-value giving the probability
    of error if the null hypothesis (equal means) is rejected at this value.
    """
    k = len(args)
    if k < 2:
        raise ValueError, "Must enter at least two input sample vectors."
    if 'equal_var' in kwds.keys():
        if kwds['equal_var']: evar = 1
        else: evar = 0
    else:
        evar = 0

    Ni = array([len(args[i]) for i in range(k)])
    Mi = array([stats.mean(args[i]) for i in range(k)])
    Vi = array([stats.var(args[i]) for i in range(k)])
    Wi = Ni / Vi
    swi = sum(Wi)
    N = sum(Ni)
    my = sum(Mi*Ni)*1.0/N
    tmp = sum((1-Wi/swi)**2 / (Ni-1.0))/(k*k-1.0)
    if evar:
        F = ((sum(Ni*(Mi-my)**2) / (k-1.0)) / (sum((Ni-1.0)*Vi) / (N-k)))
        pval = distributions.f.sf(F,k-1,N-k)  # 1-cdf
    else:
        m = sum(Wi*Mi)*1.0/swi
        F = sum(Wi*(Mi-m)**2) / ((k-1.0)*(1+2*(k-2)*tmp))
        pval = distributions.f.sf(F,k-1.0,1.0/(3*tmp))

    return F, pval



def wilcoxon(x,y=None):
    """
Calculates the Wilcoxon signed-rank test for the null hypothesis that two samples come from the same distribution. A non-parametric T-test. (need N > 20)

Returns: t-statistic, two-tailed p-value
"""
    if y is None:
        d = x
    else:
        x, y = map(asarray, (x, y))
        if len(x) <> len(y):
            raise ValueError, 'Unequal N in wilcoxon.  Aborting.'
        d = x-y
    d = compress(not_equal(d,0),d) # Keep all non-zero differences
    count = len(d)
    if (count < 10):
        print "Warning: sample size too small for normal approximation."
    r = stats.rankdata(abs(d))
    r_plus = sum((d > 0)*r)
    r_minus = sum((d < 0)*r)
    T = min(r_plus, r_minus)
    mn = count*(count+1.0)*0.25
    se = math.sqrt(count*(count+1)*(2*count+1.0)/24)
    if (len(r) != len(unique(r))):  # handle ties in data
        replist, repnum = find_repeats(r)
        corr = 0.0
        for i in range(len(replist)):
            si = repnum[i]
            corr += 0.5*si*(si*si-1.0)
        V = se*se - corr
        se = sqrt((count*V - T*T)/(count-1.0))
    z = (T - mn)/se
    prob = 2*(1.0 -stats.zprob(abs(z)))
    return T, prob

def _hermnorm(N):
    # return the negatively normalized hermite polynomials up to order N-1
    #  (inclusive)
    #  using the recursive relationship
    #  p_n+1 = p_n(x)' - x*p_n(x)
    #   and p_0(x) = 1
    plist = [None]*N
    plist[0] = poly1d(1)
    for n in range(1,N):
        plist[n] = plist[n-1].deriv() - poly1d([1,0])*plist[n-1]
    return plist

def pdf_moments(cnt):
    """Return the Gaussian expanded pdf function given the list of central
    moments (first one is mean).
    """
    N = len(cnt)
    if N < 2:
        raise ValueError, "At least two moments must be given to" + \
              "approximate the pdf."
    totp = poly1d(1)
    sig = sqrt(cnt[1])
    mu = cnt[0]
    if N > 2:
        Dvals = _hermnorm(N+1)
    for k in range(3,N+1):
        # Find Ck
        Ck = 0.0
        for n in range((k-3)/2):
            m = k-2*n
            if m % 2: # m is odd
                momdiff = cnt[m-1]
            else:
                momdiff = cnt[m-1] - sig*sig*scipy.factorial2(m-1)
            Ck += Dvals[k][m] / sig**m * momdiff
        # Add to totp 
        totp = totp +  Ck*Dvals[k]        
        
    def thisfunc(x):
        xn = (x-mu)/sig
        return totp(xn)*exp(-xn*xn/2.0)/sqrt(2*pi)/sig
    return thisfunc
           

def pdf_fromgamma(g1,g2,g3=0.0,g4=None):
    if g4 is None:
        g4 = 3*g2*g2
    sigsq = 1.0/g2
    sig = sqrt(sigsq)
    mu = g1*sig**3.0
    p12 = _hermnorm(13)
    for k in range(13):
        p12[k] = p12[k]/sig**k

    # Add all of the terms to polynomial
    totp = p12[0] - (g1/6.0*p12[3]) + \
           (g2/24.0*p12[4] +g1*g1/72.0*p12[6]) - \
           (g3/120.0*p12[5] + g1*g2/144.0*p12[7] + g1**3.0/1296.0*p12[9]) + \
           (g4/720*p12[6] + (g2*g2/1152.0+g1*g3/720)*p12[8] +
            g1*g1*g2/1728.0*p12[10] + g1**4.0/31104.0*p12[12])
    # Final normalization
    totp = totp / sqrt(2*pi)/sig
    def thefunc(x):
        xn = (x-mu)/sig
        return totp(xn)*exp(-xn*xn/2.0)
    return thefunc

def pdfapprox(samples):
    """Return a function that approximates the pdf of a set of samples
    using a Gaussian expansion computed from the mean, variance, skewness
    and Fisher's kurtosis.
    """
    # Estimate mean, variance, skewness and kurtosis
    mu,sig,sk,kur = stats.describe(samples)[2:]
    # Get central moments
    cnt = [None]*4
    cnt[0] = mu
    cnt[1] = sig*sig
    cnt[2] = sk * sig**1.5
    cnt[3] = (kur+3.0) * sig**2.0
    return pdf_moments(cnt)
    #g2 = (1.0/sig)**2.0
    #g1 = mu / sig**3.0
    #g3 = sk / sig**3.5
    #g4 = (kur+3.0) / sig**4.0
    #return pdf_fromgamma(g1, g2, g3, g4)

def circmean(samples, high=2*pi, low=0):
    """Compute the circular mean for samples assumed to be in the range [low to high]
    """
    ang = (samples - low)*2*pi / (high-low)
    res = angle(stats.mean(exp(1j*ang)))
    if (res < 0):
        res = res + 2*pi
    return res*(high-low)/2.0/pi + low

def circvar(samples, high=2*pi, low=0):
    """Compute the circular variance for samples assumed to be in the range [low to high]
    """
    ang = (samples - low)*2*pi / (high-low)
    res = stats.mean(exp(1j*ang))
    V = 1-abs(res)
    return ((high-low)/2.0/pi)**2 * V

def circstd(samples, high=2*pi, low=0):
    """Compute the circular standard deviation for samples assumed to be in the range [low to high]
    """
    ang = (samples - low)*2*pi / (high-low)
    res = stats.mean(exp(1j*ang))
    V = 1-abs(res)
    return ((high-low)/2.0/pi) * sqrt(V)
    
    
        
#Tests to include (from R) -- some of these already in stats.
########
#X Ansari-Bradley
#X Bartlett (and Levene)
#X Binomial
#Y Pearson's Chi-squared (stats.chisquare)
#Y Association Between Paired samples (stats.pearsonr, stats.spearmanr)
#                       stats.kendalltau) -- these need work though
# Fisher's exact test
#X Fligner-Killeen Test
#Y Friedman Rank Sum (stats.friedmanchisquare?)
#Y Kruskal-Wallis
#Y Kolmogorov-Smirnov
# Cochran-Mantel-Haenszel Chi-Squared for Count
# McNemar's Chi-squared for Count
#X Mood Two-Sample
#X Test For Equal Means in One-Way Layout (see stats.ttest also)
# Pairwise Comparisons of proportions
# Pairwise t tests
# Tabulate p values for pairwise comparisons
# Pairwise Wilcoxon rank sum tests
# Power calculations two sample test of prop.
# Power calculations for one and two sample t tests
# Equal or Given Proportions
# Trend in Proportions
# Quade Test
#Y Student's T Test
#Y F Test to compare two variances
#XY Wilcoxon Rank Sum and Signed Rank Tests



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

helpdict = {
   "movie" : """
   movie (draw_frame, time_limit = 120., min_interframe = 0.0,
      bracket_time = array ([2., 2.], Float ), lims = None, timing = 0)
     runs a movie based on the given DRAW_FRAME function.  The movie
     stops after a total elapsed time of TIME_LIMIT seconds, which
     defaults to 60 (one minute), or when the DRAW_FRAME function
     returns zero.

     note: All but the first argument are keyword arguments, with
     defaults as shown.

     def draw_frame(1) :
       # Input argument i is the frame number.
       # draw_frame should return non-zero if there are more
       # frames in this movie.  A zero return will stop the
       # movie.
       # draw_frame must NOT include any fma command if the
       # making_movie variable is set (movie sets this variable
       # before calling draw_frame)

     If MIN_INTERFRAME is specified, a pause will be added as
     necessary to slow down the movie.  MIN_INTERFRAME is a time
     necessary to slow down the movie.  MIN_INTERFRAME is a time
     in seconds (default 0).

     The keyword bracket_time= (again a time in seconds) can be
     used to adjust the duration of the pauses after the first
     and last frames.  It may also be a two element array [beg, end].
     If the pause at the end is greater than five seconds, you will
     be prompted to explain that hitting <RETURN> will abort the final
     pause. Well, the Python version does not have this capability.

     timing = 1 enables a timing printout for your movie.

     If every frame of your movie has the same limits, use the
     limits command to fix the limits before you call movie.

   BUG:  If you hit <RETURN> to start a movie early, it will not
         pause at the end of the movie at all.  You probably should
         not use long initial pauses.
   """,
   "movie_stats" :
   """
   movie_stats ( ) or movie_stats ( timing )
     prints statistics from the last movie command, or from the
     command which produced TIMING.  TIMING is the contents of the
     movie_timing external variable after the movie command completes.
   """,
   "'pl3d'" :
   """
   General overview of module pl3d:

   (1) Viewing transform machinery.  Arguably the simplest model
       is the CAD/CAM notion that the object you see is oriented
       as you see it in the current picture.  You can then move
       it left, right, up, down, or toward or away from you,
       or you can rotate it about any of the three axes (horizontal,
       vertical, or out of the screen).  The xyz coordinates of the
       object remains unchanged throughout all of this, but this
       object coordinate system changes relative to the fixed
       xyz of the viewer, in which x is always to the right, y is
       up, and z is directed out of the screen.  Initially, the
       two coordinate systems coincide.

       rot3 (xangle,yangle,zangle)
         Rotate the object about viewer's x-axis by xangle, then
         about viewer's y-axis by yangle, then about viewer's
         z-axis by zangle
       mov3 (xchange,ychange,zchange)
         Move the object by the specified amounts.

       setz3 (zcamera)
         The "camera" is located at (0,0,zcamera) in the viewer's
         coordinate system, looking in the minus-z direction.
         Initially, zcamera is very large, and the magnification
         factor is correspondingly large, giving an isometric view.
         Decreasing zcamera makes the perspective more extreme.
         If parts of the object are behind the camera, strange things
         may happen.

       undo3 ()
       undo3 (n)
         Undo the last N (default 1) viewpoint commands (rot3, mov3,
         or setz3).  Up to 100 viewpoint changes are remembered.
       viewpoint= save3()
       ...
       restore3 (viewpoint)
         The current viewpoint transformation can be saved and later
         restored.

       gnomon (on_off)
         Toggle the gnomon (a simple display showing the orientation
         of the xyz axes of the object).
   """,
   "set_draw3_" :
   """
   set_draw3_ ( 0 | 1 ) is used to set the global draw3_,
   which controls whether the function draw3 actually shows a drawing.
   """,
   "rot3" :
   """
   rot3 (xa, ya, za)
   rotate the current 3D plot by XA about viewer's x-axis,
   YA about viewer's y-axis, and ZA about viewer's z-axis.
   SEE ALSO: orient3, mov3, aim3, setz3, undo3, save3, restore3, light3
   """,
   "mov3" :
   """
   mov3 ( [xa [, ya [, za]]])
   move the current 3D plot by XA along the viewer's x axis,
   YA along the viewer's y axis, and ZA along the viewer's z axis.
   SEE ALSO: rot3, orient3, setz3, undo3, save3, restore3, light3
   """,
   "aim3" :
   """
   aim3 ( [xa [, ya [, za]]])
   move the current 3D plot to put the point (XA, YA, ZA) in object
   coordinates at the point (0, 0, 0) -- the aim point -- in the
   viewer's coordinates. If any of the XA, YA, or ZA is nil, it defaults
   SEE ALSO: mov3, rot3, orient3, setz3, undo3, save3, restore3, light3
   to zero.
   """,
   "setz3" :
   """
   setz3 ( [zc] )
   Set the camera position to z = ZC (x = y = 0) in the viewer's coordinate
   system. If zc is None, set the camera to infinity (default).
   SEE ALSO: rot3, orient3, undo3, save3, restore3, light3
   """,
   "orient3" :
   """
   orient3 ( [phi = val1, theta = val2] )
   Set the orientation of the object to (PHI, THETA). Orientations
   are a subset of the possible rotation matrices in which the z axis
   of the object appears vertical on the screen (that is, the object
   z axis projects onto the viewer y axis). The THETA angle is the
   angle from the viewer y axis to the object z axis, positive if
   the object z axis is tilted towards you (toward viewer +z). PHI is
   zero when the object x axis coincides with the viewer x axis. If
   neither PHI nor THETA is specified, PHI defaults to - pi / 4 and
   THETA defaults to pi / 6. If only PHI is specified, THETA remains
   unchanged, unless the current THETA is near pi / 2, in which case
   THETA returns to pi / 6, or unless the current orientation does
   not have a vertical z axis, in which case THETA returns to its
   default.
   Unlike rot3, orient3 is not a cumulative operation.
   SEE ALSO: rot3, mov3, aim3, save3, restore3, light3
   """,
   "save3" :
   """
   view = save3 ( )
     Save the current 3D viewing transformation and lighting.
     Actually, this doesn't save anything; it returns a copy
     of the current 3D viewing transformation and lighting, so
     that the user can put it aside somewhere.
   SEE ALSO: restore3, rot3, mov3, aim3, light3
   """,
   "restore3" :
   """
   restore3 ( view )
   Restore a previously saved 3D viewing transformation and lighting.
   If view is missing, rotate object to viewer's coordinate system.
   SEE ALSO: restore3, rot3, mov3, aim3, light3
   """,
   "light3" :
   """
   light3 (ambient=a_level,
                    diffuse=d_level,
                    specular=s_level,
                    spower=n,
                    sdir=xyz)
     Sets lighting properties for 3D shading effects.
     A surface will be shaded according to its to its orientation
     relative to the viewing direction.

     The ambient level A_LEVEL is a light level (arbitrary units)
     that is added to every surface independent of its orientation.

     The diffuse level D_LEVEL is a light level which is proportional
     to cos(theta), where theta is the angle between the surface
     normal and the viewing direction, so that surfaces directly
     facing the viewer are bright, while surfaces viewed edge on are
     unlit (and surfaces facing away, if drawn, are shaded as if they
     faced the viewer).

     The specular level S_LEVEL is a light level proportional to a high
     power spower=N of 1+cos(alpha), where alpha is the angle between
     the specular reflection angle and the viewing direction.  The light
     source for the calculation of alpha lies in the direction XYZ (a
     3 element vector) in the viewer's coordinate system at infinite
     distance.  You can have ns light sources by making S_LEVEL, N, and
     XYZ (or any combination) be vectors of length ns (3-by-ns in the
     case of XYZ).  (See source code for specular_hook function
     definition if powers of 1+cos(alpha) aren't good enough for you.)

     With no arguments, return to the default lighting.

   EXAMPLES:
     light3 ( diffuse=.1, specular=1., sdir=[0,0,-1])
       (dramatic "tail lighting" effect)
     light3 ( diffuse=.5, specular=1., sdir=[1,.5,1])
       (classic "over your right shoulder" lighting)
     light3 ( ambient=.1,diffuse=.1,specular=1.,
             sdir=[[0,0,-1],[1,.5,1]],spower=[4,2])
       (two light sources combining previous effects)
   SEE ALSO: rot3, save3, restore3
   """,
   "get3_light" :
   """
   get3_light(xyz, nxyz)
      or get3_light(xyz)

     return 3D lighting for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len (NXYZ) long; in the second case, the
     return value is (ni-1)-by-(nj-1).

     The parameters of the lighting calculation are set by the
     light3 function.

     SEE ALSO: light3, set3_object, get3_normal, get3_centroid
   """,
   "get3_normal" :
   """
     get3_normal(xyz, nxyz)
         or get3_normal(xyz)

     return 3D normals for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len(NXYZ)-by-3; in the second case, the
     return value is (ni-1)-by-(nj-1)-by-3.

     The normals are constructed from the cross product of the lines
     joining the midpoints of two edges which as nearly quarter the
     polygon as possible (the medians for a quadrilateral).  No check
     is made that these not be parallel; the returned "normal" is
     [0,0,0] in that case.  Also, if the polygon vertices are not
     coplanar, the "normal" has no precisely definable meaning.

     SEE ALSO: get3_centroid, get3_light
   """,
   "get3_centroid" :
   """
     get3_centroid(xyz, *nxyz)
         or get3_centroid(xyz)

     return 3D centroids for polygons with vertices XYZ.  If NXYZ is
     specified, XYZ should be sum(nxyz)-by-3, with NXYZ being the
     list of numbers of vertices for each polygon (as for the plfp
     function).  If NXYZ is not specified, XYZ should be a quadrilateral
     mesh, ni-by-nj-by-3 (as for the plf function).  In the first case,
     the return value is len(NXYZ) in length; in the second case, the
     return value is (ni-1)-by-(nj-1)-by-3.

     The centroids are constructed as the mean value of all vertices
     of each polygon.

     SEE ALSO: get3_normal, get3_light
   """,
   "get3_xy" :
   """
     get3_xy (xyz)
         or get3_xy(xyz, 1)

     Given anything-by-3 coordinates XYZ, return X and Y in viewer's
     coordinate system (set by rot3, mov3, orient3, etc.).  If the
     second argument is present and non-zero, also return Z (for use
     in sort3d or get3_light, for example).  If the camera position
     has been set to a finite distance with setz3, the returned
     coordinates will be tangents of angles for a perspective
     drawing (and Z will be scaled by 1/zc).
     Unlike the Yorick version, this function returns a 3-by-anything
     array of coordinates.
     Actually, what it returns is a 3-by-anything python array, whose
     0th element is the x array, whose 1th element is the y array, and
     whose 2th element is the z array if asked for.
     I believe that x, y, and z can be either 1d or 2d, so this
     routine is written in two cases.
   """,
   "undo3" :
   """
     undo3 ()
         or undo3 (n)
     Undo the effects of the last N (default 1) rot3, orient3, mov3, aim3,
     setz3, or light3 commands.
   """,
   "set3_object" :
   """
     set3_object (drawing_function, [arg1,arg2,...])

     set up to trigger a call to draw3, adding a call to the
     3D display list of the form:

        DRAWING_FUNCTION ( [ARG1, ARG2, ...]))

     When draw3 calls DRAWING_FUNCTION, the external variable draw3_
     will be non-zero, so DRAWING_FUNCTION can be written like this:

     def drawing_function(arg) :
      
       if (draw3_) :
          arg1= arg [0]
          arg1= arg [1]
          ...
          ...<calls to get3_xy, sort3d, get3_light, etc.>...
          ...<calls to graphics functions plfp, plf, etc.>...
          return

       ...<verify args>...
       ...<do orientation and lighting independent calcs>...
       set3_object (drawing_function, [arg1,arg2,...])

   SEE ALSO: get3_xy, get3_light, sort3d
   """,
   "window3" :
   """
   window3 ( ) or window3 (n)
   initialize style="nobox.gs" window for 3D graphics
   """,
   "sort3d" :
   """
   sort3d(z, npolys)
     given Z and NPOLYS, with len(Z)==sum(npolys), return
     a 2-element list [LIST, VLIST] such that Z[VLIST] and NPOLYS[LIST] are
     sorted from smallest average Z to largest average Z, where
     the averages are taken over the clusters of length NPOLYS.
     Within each cluster (polygon), the cyclic order of Z[VLIST]
     remains unchanged, but the absolute order may change.

     This sorting order produces correct or nearly correct order
     for a plfp command to make a plot involving hidden or partially
     hidden surfaces in three dimensions.  It works best when the
     polys form a set of disjoint closed, convex surfaces, and when
     the surface normal changes only very little between neighboring
     polys.  (If the latter condition holds, then even if sort3d
     mis-orders two neighboring polys, their colors will be very
     nearly the same, and the mistake won't be noticeable.)  A truly
     correct 3D sorting routine is impossible, since there may be no
     rendering order which produces correct surface hiding (some polys
     may need to be split into pieces in order to do that).  There
     are more nearly correct algorithms than this, but they are much
     slower.
   SEE ALSO: get3_xy
   """,
   "draw3" :
   """
      draw3 (called_as_idler = 0, lims = None):
   Draw the current 3d display list.
   Ordinarily triggered automatically when the drawing changes.
   """,
   "gnomon" :
   """
   gnomon ()
      or gnomon (onoff)
     Toggle the gnomon display. If on is present and non-zero,
     turn on the gnomon. If zero, turn it off.

     The gnomon shows the X, Y, and Z axis directions in the
     object coordinate system. The directions are labeled.
     The gnomon is always infinitely far behind the object
     (away from the camera).

     There is a mirror-through-the-screen-plane ambiguity in the
     display which is resolved in two ways: (1) the (X, Y, Z)
     coordinate system is right-handed, and (2) If the tip of an
     axis projects into the screen, its label is drawn in opposite
     polarity to the other text in the screen.
   """,
   "spin3" :
   """
   spin3 ( ) or spin3 (nframes) os spin3 (nframes, axis)
     Spin the current 3D display list about AXIS over NFRAMES.  Keywords
     tlimit= the total time allowed for the movie in seconds (default 60),
     dtmin= the minimum allowed interframe time in seconds (default 0.0),
     bracket_time= (as for movie function in movie.i), timing = 1 if
     you want timing measured and printed out, 0 if not.

     The default AXIS is [-1,1,0] and the default NFRAMES is 30.
   SEE ALSO: rot3
   """,
   "plwf" :
   """
   plwf (z)
   or plwf (z, y, x)

     plots a 3-D wire frame of the given Z array, which must have the
     same dimensions as the mesh (X, Y).  If X and Y are not given, they
     default to the first and second indices of Z, respectively.
     The drawing order of the zones is determined by a simple "painter's
     algorithm", which works fairly well if the mesh is reasonably near
     rectilinear, but can fail even then if the viewpoint is chosen to
     produce extreme fisheye perspective effects.  Look at the resulting
     plot carefully to be sure the algorithm has correctly rendered the
     model in each case.

   KEYWORDS: fill   -- optional colors to use (default is to make zones
                       have background color), same dimension options as
                       for z argument to plf function
             shade  -- set non-zero to compute shading from current
                       3D lighting sources
             edges  -- default is 1 (draw edges), but if you provide fill
                       colors, you may set to 0 to supress the edges
             ecolor, ewidth  -- color and width of edges
             cull   -- default is 1 (cull back surfaces), but if you want
                       to see the "underside" of the model, set to 0
             scale  -- by default, Z is scaled to "reasonable" maximum
                       and minimum values related to the scale of (X,Y).
                       This keyword alters the default scaling factor, in
                       the sense that scale=2.0 will produce twice the
                       Z-relief of the default scale=1.0.
             cmax   -- the ambient= keyword in light3 can be used to
                       control how dark the darkest surface is; use this
                       to control how light the lightest surface is
                       the lightwf routine can change this parameter
                       interactively

   SEE ALSO: lightwf, plm, plf, orient3, light3, fma3, window3
   """,
   "lightwf" :
   """
   lightwf (cmax)
     Sets the cmax= parameter interactively, assuming the current
     3D display list contains the result of a previous plwf call.
     This changes the color of the brightest surface in the picture.
     The darkest surface color can be controlled using the ambient=
     keyword to light3.

   SEE ALSO: plwf, light3
   """,
   "xyz_wf" :
   """
   xyz_wf (z, [y, x] [,scale = 1.0])
      returns a 3-by-ni-by-nj array whose 0th entry is x, 1th entry
      is y, and 2th entry is z. z is ni-by-nj. x and y, if present,
      must be the same shape. If not present, integer ranges will
      be used to create an equally spaced coordinate grid in x and y.
      The function which scales the "topography" of z(x,y) is
      potentially useful apart from plwf.
      For example, the xyz array used by plwf can be converted from
      a quadrilateral mesh plotted using plf to a polygon list plotted
      using plfp like this:
        xyz= xyz_wf(z,y,x,scale=scale);
        ni= shape(z)[1];
        nj= shape(z)[2];
        list = ravel (add.outer (
           ravel(add.outer (adders,zeros(nj-1, Int))) +
           arange((ni-1)*(nj-1), typecode = Int),
           array ( [[0, 1], [nj + 1, nj]])))
        xyz=array([take(ravel(xyz[0]),list),
           take(ravel(xyz[1]),list),
           take(ravel(xyz[2]),list)])
        nxyz= ones((ni-1)*(nj-1)) * 4;
      The resulting array xyz is 3-by-(4*(nj-1)*(ni-1)).
      xyz[0:3,4*i:4*(i+1)] are the clockwise coordinates of the
      vertices of cell number i.
   """,
   "plane3" :
   """
   plane3(normal, point)
         or plane3([nx,ny,nz], [px,py,pz])

     returns [nx,ny,nz,pp] for the specified plane.
   """,
   "mesh3" :
   """
    mesh3(x,y,z)
         or mesh3(x,y,z, funcs = [f1,f2,...])
         or mesh3(xyz, funcs = [f1,f2,...])
         or mesh3(nxnynz, dxdydz, x0y0z0, funcs = [f1,f2,...])

     make mesh3 argument for slice3, xyz3, getv3, etc., functions.
     X, Y, and Z are each 3D coordinate arrays.  The optional F1, F2,
     etc. are 3D arrays of function values (e.g. density, temperature)
     which have one less value along each dimension than the coordinate
     arrays.  The "index" of each zone in the returned mesh3 is
     the index in these cell-centered Fi arrays, so every index from
     one through the total number of cells indicates one real cell.
     The Fi arrays can also have the same dimensions as X, Y, or Z
     in order to represent point-centered quantities.

     If X has four dimensions and the length of the first is 3, then
     it is interpreted as XYZ (which is the quantity actually stored
     in the returned cell list).

     If X is a vector of 3 integers, it is interpreted as [nx,ny,nz]
     of a uniform 3D mesh, and the second and third arguments are
     [dx,dy,dz] and [x0,y0,z0] respectively.  (DXDYDZ represent the
     size of the entire mesh, not the size of one cell, and NXNYNZ are
     the number of cells, not the number of points.)

     Added by ZCM 1/13/97: if x, y, and z are one-dimensional of
     the same length and if the keyword verts exists and yields
     an NCELLS by 8 integer array, then we have an unstructured
     rectangular mesh, and the subscripts of cell i's vertices
     are verts[i, 0:8].

     Other sorts of meshes are possible -- a mesh which lives
     in a binary file is an obvious example -- which would need
     different workers for xyz3, getv3, getc3, and iterator3
     iterator3_rect may be more general than the other three;
     as long as the cell dimensions are the car of the list
     which is the 2nd car of m3, it will work.
   """,
   "slice3" :
   """
   slice3 (m3, fslice, nverts, xyzverts)
         or color_values= slice3(m3, fslice, nverts, xyzverts, fcolor)
         or color_values= slice3(m3, fslice, nverts, xyzverts, fcolor, 1)

     slice the 3D mesh M3 using the slicing function FSLICE, returning
     the list [NVERTS, XYZVERTS, color].  Note that it is impossible to
     pass arguments as addresses, as yorick does in this routine.
     NVERTS is the number of vertices in each polygon of the slice, and
     XYZVERTS is the 3-by-sum(NVERTS) list of polygon vertices.  If the
     FCOLOR argument is present, the values of that coloring function on
     the polygons are returned as the value of the slice3 function
     (numberof(color_values) == numberof(NVERTS) == number of polygons).

     If the slice function FSLICE is a function, it should be of the
     form:
        func fslice(m3, chunk)
     returning a list of function values on the specified chunk of the
     mesh m3.  The format of chunk depends on the type of m3 mesh, so
     you should use only the other mesh functions xyz3 and getv3 which
     take m3 and chunk as arguments.  The return value of fslice should
     have the same dimensions as the return value of getv3; the return
     value of xyz3 has an additional first dimension of length 3.

     If FSLICE is a list of 4 numbers, it is taken as a slicing plane
     with the equation FSLICE(+:1:3)*xyz(+)-FSLICE(4), as returned by
     plane3.

     If FSLICE is a single integer, the slice will be an isosurface for
     the FSLICEth variable associated with the mesh M3.  In this case,
     the keyword value= must also be present, representing the value
     of that variable on the isosurface.

     If FCOLOR is nil, slice3 returns nil.  If you want to color the
     polygons in a manner that depends only on their vertex coordinates
     (e.g.- by a 3D shading calculation), use this mode.

     If FCOLOR is a function, it should be of the form:
        func fcolor(m3, cells, l, u, fsl, fsu, ihist)
     returning a list of function values on the specified cells of the
     mesh m3.  The cells argument will be the list of cell indices in
     m3 at which values are to be returned.  l, u, fsl, fsu, and ihist
     are interpolation coefficients which can be used to interpolate
     from vertex centered values to the required cell centered values,
     ignoring the cells argument.  See getc3 source code.
     The return values should always have dimsof(cells).

     If FCOLOR is a single integer, the slice will be an isosurface for
     the FCOLORth variable associated with the mesh M3.

     If the optional argument after FCOLOR is non-nil and non-zero,
     then the FCOLOR function is called with only two arguments:
        func fcolor(m3, cells)
   """,
   "slice3mesh" :
   """
   slice3mesh returns a triple [nverts, xyzverts, color]
    nverts is no_cells long and the ith entry tells how many
       vertices the ith cell has.
    xyzverts is sum (nverts) by 3 and gives the vertex
       coordinates of the cells in order.
    color, if present, is len (nverts) long and contains
       a color value for each cell in the mesh.

   There are a number of ways to call slice3mesh:
 
      slice3mesh (z, color = None)

   z is a two dimensional array of cell function values, assumed
      to be on a uniform mesh nx by ny cells (assuming z is nx by ny)
      nx being the number of cells in the x direction, ny the number
      in the y direction.
   color, if specified, is either an nx by ny array
      of cell-centered values by which the surface is to
      be colored, or an nx +1 by ny + 1 array of vertex-
      centered values, which will be averaged over each
      cell to give cell-centered values.

      slice3mesh (nxny, dxdy, x0y0, z, color = None)

   In this case, slice3mesh accepts the specification for
   a regular 2d mesh: nxny is the number of cells in the
   x direction and the y direction; x0y0 are the initial
   values of x and y; and dxdy are the increments in the
   two directions. z is the height of a surface above
   the xy plane and must be dimensioned nx + 1 by ny + 1.
   color, if specified, is as above.

      slice3mesh (x, y, z, color = None)

   z is as above, an nx by ny array of function values
   on a mesh of the same dimensions. There are two choices
   for x and y: they can both be one-dimensional, dimensioned
   nx and ny respectively, in which case they represent a
   mesh whose edges are parallel to the axes; or else they
   can both be nx by ny, in which case they represent a
   general quadrilateral mesh.
   color, if specified, is as above.
   """,
   "iterator3" :
   """
   iterator3 (m3)
   iterator3 (m3, chunk, clist)
   iterator3_rect (m3)
   iterator3_rect (m3, chunk, clist)
   iterator3_irreg (m3)
   iterator3_irreg (m3, chunk, clist)

   The iterator3 functions combine three distinct operations:
   (1) If only the M3 argument is given, return the initial
       chunk of the mesh.  The chunk will be no more than
       chunk3_limit cells of the mesh.
   (2) If only M3 and CHUNK are given, return the next CHUNK,
       or None if there are no more chunks.
   (3) If M3, CHUNK, and CLIST are all specified, return the
       absolute cell index list corresponding to the index list
       CLIST of the cells in the CHUNK.
       Do not increment the chunk in this case.

   The form of the CHUNK argument and return value for cases (1)
   and (2) is not specified, but it must be recognized by the
   xyz3 and getv3 functions which go along with this iterator3.
   (For case (3), CLIST and the return value are both ordinary
   index lists.)
   In the irregular case, it is guaranteed that the returned chunk
   consists of only one type of cell (tetrahedra, hexahedra,
   pyramids, or prisms).
   """,
   "getv3" :
   """
   getv3(i, m3, chunk)

     return vertex values of the Ith function attached to 3D mesh M3
     for cells in the specified CHUNK.  The CHUNK may be a list of
     cell indices, in which case getv3 returns a 2x2x2x(dimsof(CHUNK))
     list of vertex coordinates.  CHUNK may also be a mesh-specific data
     structure used in the slice3 routine, in which case getv3 may
     return a (ni)x(nj)x(nk) array of vertex values.  For meshes which
     are logically rectangular or consist of several rectangular
     patches, this is up to 8 times less data, with a concomitant
     performance advantage.  Use getv3 when writing slicing functions
     for slice3.
   getv3_rect(i, m3, chunk) does the job for a regular rectangular
     mesh.
   getv3_irreg (i, m3, chunk) :
     for an irregular mesh, returns a 3-list whose elements are:
     (1) the function values for the ith function on the vertices of the
     given chunk. (The function values must have the same dimension
     as the coordinates; there is no attempt to convert zone-centered
     values to vertex-centered values.)
     (2) an array of relative cell numbers within the list of cells
     of this type.
     (3) a number that can be added to these relative numbers to gives
     the absolute cell numbers for correct access to their coordinates
     and function values.
   """,
   "getc3" :
   """
   getc3(i, m3, chunk)
         or getc3(i, m3, clist, l, u, fsl, fsu, cells)

     return cell values of the Ith function attached to 3D mesh M3
     for cells in the specified CHUNK.  The CHUNK may be a list of
     cell indices, in which case getc3 returns a (dimsof(CHUNK))
     list of vertex coordinates.  CHUNK may also be a mesh-specific data
     structure used in the slice3 routine, in which case getc3 may
     return a (ni)x(nj)x(nk) array of vertex values.  There is no
     savings in the amount of data for such a CHUNK, but the gather
     operation is cheaper than a general list of cell indices.
     Use getc3 when writing colorng functions for slice3.

     If CHUNK is a CLIST, the additional arguments L, U, FSL, and FSU
     are vertex index lists which override the CLIST if the Ith attached
     function is defined on mesh vertices.  L and U are index lists into
     the (dimsof(CLIST))x2x2x2 vertex value array, say vva, and FSL
     and FSU are corresponding interpolation coefficients; the zone
     centered value is computed as a weighted average of involving these
     coefficients.  The CELLS argument is required by histogram to do
     the averaging.  See the source code for details.
     By default, this conversion (if necessary) is done by averaging
     the eight vertex-centered values.
   getc3_rect (i, m3, chunk, l, u, fsl, fsu, cells) does the job
     for a regular rectangular mesh.
   getc3_irreg (i, m3, chunk, l, u, fsl, fsu, cells) :
      Same thing as getc3_rect, i. e., returns the same type of
      data structure, but from an irregular mesh.
      m3 [1] is a 2-list; m3[1] [0] is an array whose ith element
         is an array of coordinate indices for the ith cell,
         or a list of up to four such arrays.
         m3 [1] [1] is the 3 by nverts array of coordinates.
      m3 [2] is a list of arrays of vertex-centered or cell-centered
         data.
   chunk may be a list, in which case chunk [0] is a 2-sequence
      representing a range of cell indices; or it may be a one-dimensional
      array, in which case it is a nonconsecutive set of cell indices.
      It is guaranteed that all cells indexed by the chunk are the
      same type.
   """,
   "slice2x" :
   """
   slice2x (plane, nverts, xyzverts, values)

     Slice a polygon list, retaining only those polygons or
     parts of polygons on the positive side of PLANE, that is,
     the side where xyz(+)*PLANE(+:1:3)-PLANE(4) > 0.0.
     The NVERTS, VALUES, and XYZVERTS arrays have the meanings of
     the return values from the slice3 function.
     Python returns a sextuple
     [nverts, xyzverts, values, nvertb, xyzvertb, valueb]
     with None in the place of missing or None input arguments.

   slice2_precision= precision
     Controls how slice2 (or slice2x) handles points very close to
     the slicing plane.  PRECISION should be a positive number or zero.
     Zero PRECISION means to clip exactly to the plane, with points
     exactly on the plane acting as if they were slightly on the side
     the normal points toward.  Positive PRECISION means that edges
     are clipped to parallel planes a distance PRECISION on either
     side of the given plane.  (Polygons lying entirely between these
     planes are completely discarded.)

     Default value is 0.0.
   """,
   "slice2" :
   """
   slice2 (plane, nverts, xyzverts, values)

     Slice a polygon list, retaining only those polygons or
     parts of polygons on the positive side of PLANE, that is,
     the side where xyz(+)*PLANE(+:1:3)-PLANE(4) > 0.0.
     The NVERTS, VALUES, and XYZVERTS arrays have the meanings of
     the return values from the slice3 function.
     Python returns a sextuple
     [nverts, xyzverts, values, nvertb, xyzvertb, valueb]
     with None in the place of missing or None input arguments.
     It is legal for the VALUES argument to be None (e.g.- if there
     is no fcolor function).

     In order to plot two intersecting slices, one could
     slice (for example) the horizontal plane twice (slice2x) -
     first with the plane of the vertical slice, then with minus
     that same plane.  Then, plot first the back part of the
     slice, then the vertical slice, then the front part of the
     horizontal slice.  Of course, the vertical plane could
     be the one to be sliced, and "back" and "front" vary
     depending on the view point, but the general idea always
     works.

   slice2_precision= precision
     Controls how slice2 (or slice2x) handles points very close to
     the slicing plane.  PRECISION should be a positive number or zero.
     Zero PRECISION means to clip exactly to the plane, with points
     exactly on the plane acting as if they were slightly on the side
     the normal points toward.  Positive PRECISION means that edges
     are clipped to parallel planes a distance PRECISION on either
     side of the given plane.  (Polygons lying entirely between these
     planes are completely discarded.)

     Default value is 0.0.
   """,
   "pl3surf" :
   """
   pl3surf (nverts, xyzverts)
         or pl3surf (nverts, xyzverts, values)

     Perform simple 3D rendering of an object created by slice3
     (possibly followed by slice2).  NVERTS and XYZVERTS are polygon
     lists as returned by slice3, so XYZVERTS is sum(NVERTS)-by-3,
     where NVERTS is a list of the number of vertices in each polygon.
     If present, the VALUES should have the same length as NVERTS;
     they are used to color the polygon.  If VALUES is not specified,
     the 3D lighting calculation set up using the light3 function
     will be carried out.  Keywords cmin= and cmax= as for plf, pli,
     or plfp are also accepted.  (If you do not supply VALUES, you
     probably want to use the ambient= keyword to light3 instead of
     cmin= here, but cmax= may still be useful.)
   """,
   "pl3tree" :
   """
   pl3tree (nverts, xyzverts = None, values = None, plane = None,
      cmin = None, cmax = None)
   
     Add the polygon list specified by NVERTS (number of vertices in
     each polygon) and XYZVERTS (3-by-sum(NVERTS) vertex coordinates)
     to the currently displayed b-tree.  If VALUES is specified, it
     must have the same dimension as NVERTS, and represents the color
     of each polygon.  If VALUES is not specified, the polygons
     are assumed to form an isosurface which will be shaded by the
     current 3D lighting model; the isosurfaces are at the leaves of
     the b-tree, sliced by all of the planes.  If PLANE is specified,
     the XYZVERTS must all lie in that plane, and that plane becomes
     a new slicing plane in the b-tree.

     Each leaf of the b-tree consists of a set of sliced isosurfaces.
     A node of the b-tree consists of some polygons in one of the
     planes, a b-tree or leaf entirely on one side of that plane, and
     a b-tree or leaf on the other side.  The first plane you add
     becomes the root node, slicing any existing leaf in half.  When
     you add an isosurface, it propagates down the tree, getting
     sliced at each node, until its pieces reach the existing leaves,
     to which they are added.  When you add a plane, it also propagates
     down the tree, getting sliced at each node, until its pieces
     reach the leaves, which it slices, becoming the nodes closest to
     the leaves.

     tree is a 4-element list like this:
      [plane, back_tree, inplane_leaf, front_tree]
      plane= tree [0]  is None if this is just a leaf
                       in which case, only inplane_leaf is not None
      back_tree= tree [1]    is the part behind plane
      inplane_leaf= tree [2] is the part in the plane itself
      front_tree= tree [3]   is the part in front of plane

     This structure is relatively easy to plot, since from any
     viewpoint, a node can always be plotted in the order from one
     side, then the plane, then the other side.

     This routine assumes a "split palette"; the colors for the
     VALUES will be scaled to fit from color 0 to color 99, while
     the colors from the shading calculation will be scaled to fit
     from color 100 to color 199.  (If VALUES is specified as a char
     array, however, it will be used without scaling.)
     You may specifiy a cmin= or cmax= keyword to affect the
     scaling; cmin is ignored if VALUES is not specified (use the
     ambient= keyword from light3 for that case).
   """,
   "split_palette" :
   """
   split_palette
         or split_palette ("palette_name.gp")
     split the current palette or the specified palette into two
     parts; colors 0 to 99 will be a compressed version of the
     original, while colors 100 to 199 will be a gray scale.
   """,
   "split_bytscl" :
   """
   split_bytscl (x, upper, cmin = None, cmax = None)
     as bytscl function, but scale to the lower half of a split
     palette (0-99, normally the color scale) if the second parameter
     is zero or nil, or the upper half (100-199, normally the gray
     scale) if the second parameter is non-zero.
   """,
   "xyz3" :
   """
   xyz3 (m3, chunk)

     return vertex coordinates for CHUNK of 3D mesh M3.  The CHUNK
     may be a list of cell indices, in which case xyz3 returns a
     (dimsof(CHUNK))x3x2x2x2 list of vertex coordinates.  CHUNK may
     also be a mesh-specific data structure used in the slice3
     routine, in which case xyz3 may return a 3x(ni)x(nj)x(nk)
     array of vertex coordinates.  For meshes which are logically
     rectangular or consist of several rectangular patches, this
     is up to 8 times less data, with a concomitant performance
     advantage.  Use xyz3 when writing slicing functions or coloring
     functions for slice3.
   """,
   "to_corners3" :
   """
   to_corners3(list, nj, nk)
     convert an array of cell indices in an (ni-1)-by-(NJ-1)-by-(NK-1)
     logically rectangular grid of cells into the list of
     len(LIST)-by-2-by-2-by-2 cell corner indices in the
     corresponding ni-by-NJ-by-NK array of vertices.
     Note that this computation in Yorick gives an absolute offset
     for each cell quantity in the grid. In Yorick it is legal to
     index a multidimensional array with an absolute offset. In
     Python it is not. However, an array can be flattened if
     necessary.
     Other changes from Yorick were necessitated by row-major
     order and 0-origin indices, and of course the lack of
     Yorick array facilities.
   """,
   "'yorick'" :
   """
   The yorick module supplies Python versions of some common
   yorick functions: zcen_, dif_, maxelt_, minelt_, rem_0_,
   avg_, timer_, timer_print.
   """,
   "zcen_" :
   """
   zcen_(x, i) does the same thing as in Yorick: x(...,zcen,...)
   where zcen is the ith subscript. (works for up to 5 dimensions).
   Namely, the elements along the ith dimension of x are replaced
   by the averages of adjacent pairs, and the dimension decreases
   by one. Remember that Python sunscripts are counted from 0.
   """,
   "dif_" :
   """
   dif_(x, i) does the same thing as in Yorick: x(...,dif_,...)
   where dif_ is the ith subscript. (works for up to 5 dimensions).
   Namely, the elements along the ith dimension of x are replaced
   by the differences of adjacent pairs, and the dimension decreases
   by one. Remember that Python sunscripts are counted from 0.
   """,
   "maxelt_" :
   """
   maxelt_ accepts a sequence of one or more possible multi-dimensional
   objects and computes their maximum. In principle these can be of
   arbitrary complexity, since the routine recurses.
   """,
   "minelt_" :
   """
   minelt_ accepts a sequence of one or more possible multi-dimensional
   objects and computes their minimum. In principle these can be of
   arbitrary complexity, since the routine recurses.
   """,
   "rem_0_" :
   """
   rem_0_ (z) goes through array z and replaces any zero
   elements with 1.e-35. Assumes z has one or two dimensions.
   """,
   "avg_" :
   """
   avg_ (z) returns the average of all elements of its array
   argument.
   """,
   "timer_" :
   """
   timer (elapsed) returns a triple consisting of the times
   [cpu, system, wall].
   timer (elapsed, split) returns a sequence whose first element
   is [cpu, system, wall] and whose second element is the
   sum of split and the difference between ththe new and old values
   of 'elapsed.'
   """,
   "timer_print" : 
   """
   timer_print (label1, split1 [,label2, split2, ...]) prints
   out a timing summary for splits accumulated by timer_.
   """
}

from string import *

def gist3dhelp ( routine ) :
   routine = `routine` [0:min (len (`routine`), 58)]
   key = None
   lenkey = 0
   for k in helpdict.keys () :
      if find (routine, k) >= 0 :
         if len (k) > lenkey :
            key = k
            lenkey = len (k)
   if key is not None:
      print helpdict [key]
   else :
      print "No help available for " + routine + "."
   return



#
# plt - Plotting with wxPython
#

from info_plt import __doc__

from lena import lena

from plot_utility import *
from interface import *

# XXX: why we need this?:
import plot_utility
import interface

from scipy_base import display_test
_have_wx = 0
if not display_test.have_x11() or display_test.try_XOpenDisplay():
    try:
        import wxPython
        _have_wx = 1
    except ImportError,msg:
        print __file__,msg

if _have_wx:
    from plot_objects import *
    from wxplt import *
    import wxplt
    import plot_objects





#!/usr/bin/env python

from __future__ import nested_scopes
import os
import glob

def configuration(parent_package='',parent_path=None):
    from scipy_distutils.core import Extension
    from scipy_distutils.misc_util import get_path, default_config_dict, dot_join
    from scipy_distutils.system_info import get_info,dict_append,NotFoundError

    package = 'integrate'
    config = default_config_dict(package,parent_package)
    local_path = get_path(__name__,parent_path)
    def local_join(*paths):
        return os.path.join(*((local_path,)+paths))
    def local_glob(*names):
        return glob.glob(os.path.join(*((local_path,)+names)))

    blas_opt = get_info('blas_opt')
    if not blas_opt:
        raise NotFoundError,'no blas resources found'

    linpack_lite = ('linpack_lite_src',{'sources':local_glob('linpack_lite','*.f')})
    mach = ('mach_src',{'sources':local_glob('mach','*.f')})
    quadpack = ('quadpack_src',{'sources':local_glob('quadpack','*.f')})
    odepack = ('odepack_src',{'sources':local_glob('odepack','*.f')})

    # should we try to weed through files and replace with calls to
    # LAPACK routines?
    # Yes, someday...

    # Extensions
    # quadpack:
    ext_args = {}
    dict_append(ext_args,
                name=dot_join(parent_package,package,'_quadpack'),
                sources = [local_join('_quadpackmodule.c')],
                libraries = [quadpack,linpack_lite,mach],
                )
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # odepack
    ext_args = {}
    dict_append(ext_args,
                name=dot_join(parent_package,package,'_odepack'),
                sources = [local_join('_odepackmodule.c')],
                libraries = [odepack,linpack_lite,mach],
                )
    dict_append(ext_args,**blas_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    # vode
    ext_args = {}
    dict_append(ext_args,
                name=dot_join(parent_package,package,'vode'),
                sources = [local_join('vode.pyf')],
                libraries = [odepack,linpack_lite,mach],
                )
    dict_append(ext_args,**blas_opt)
    ext = Extension(**ext_args)
    config['ext_modules'].append(ext)

    return config

if __name__ == '__main__':    
    from scipy_distutils.core import setup
    setup(**configuration(parent_path=''))
